# Copyright (C) 2012 Nippon Telegraph and Telephone Corporation.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging

import json
import ast
from webob import Response

from ryu.base import app_manager
from ryu.controller import ofp_event
from ryu.controller import dpset
from ryu.controller.handler import MAIN_DISPATCHER
from ryu.controller.handler import set_ev_cls
from ryu.ofproto import ofproto_v1_0
from ryu.ofproto import ofproto_v1_2
from ryu.ofproto import ofproto_v1_3
from ryu.lib import ofctl_v1_0
from ryu.lib import ofctl_v1_2
from ryu.lib import ofctl_v1_3
from ryu.app.wsgi import ControllerBase, WSGIApplication


LOG = logging.getLogger('ryu.app.ofctl_rest')

# supported ofctl versions in this restful app
supported_ofctl = {
    ofproto_v1_0.OFP_VERSION: ofctl_v1_0,
    ofproto_v1_2.OFP_VERSION: ofctl_v1_2,
    ofproto_v1_3.OFP_VERSION: ofctl_v1_3,
}

# REST API
#

# Retrieve the switch stats
#
# get the list of all switches
# GET /stats/switches
#
# get the desc stats of the switch
# GET /stats/desc/<dpid>
#
# get flows stats of the switch
# GET /stats/flow/<dpid>
#
# get flows stats of the switch filtered by the fields
# POST /stats/flow/<dpid>
#
# get aggregate flows stats of the switch
# GET /stats/aggregateflow/<dpid>
#
# get aggregate flows stats of the switch filtered by the fields
# POST /stats/aggregateflow/<dpid>
#
# get table stats of the switch
# GET /stats/table/<dpid>
#
# get table features stats of the switch
# GET /stats/tablefeatures/<dpid>
#
# get ports stats of the switch
# GET /stats/port/<dpid>
#
# get queues stats of the switch
# GET /stats/queue/<dpid>
#
# get queues config stats of the switch
# GET /stats/queueconfig/<dpid>/<port>
#
# get meter features stats of the switch
# GET /stats/meterfeatures/<dpid>
#
# get meter config stats of the switch
# GET /stats/meterconfig/<dpid>
#
# get meters stats of the switch
# GET /stats/meter/<dpid>
#
# get group features stats of the switch
# GET /stats/groupfeatures/<dpid>
#
# get groups desc stats of the switch
# GET /stats/groupdesc/<dpid>
#
# get groups stats of the switch
# GET /stats/group/<dpid>
#
# get ports description of the switch
# GET /stats/portdesc/<dpid>

# Update the switch stats
#
# add a flow entry
# POST /stats/flowentry/add
#
# modify all matching flow entries
# POST /stats/flowentry/modify
#
# modify flow entry strictly matching wildcards and priority
# POST /stats/flowentry/modify_strict
#
# delete all matching flow entries
# POST /stats/flowentry/delete
#
# delete flow entry strictly matching wildcards and priority
# POST /stats/flowentry/delete_strict
#
# delete all flow entries of the switch
# DELETE /stats/flowentry/clear/<dpid>
#
# add a meter entry
# POST /stats/meterentry/add
#
# modify a meter entry
# POST /stats/meterentry/modify
#
# delete a meter entry
# POST /stats/meterentry/delete
#
# add a group entry
# POST /stats/groupentry/add
#
# modify a group entry
# POST /stats/groupentry/modify
#
# delete a group entry
# POST /stats/groupentry/delete
#
# modify behavior of the physical port
# POST /stats/portdesc/modify
#
#
# send a experimeter message
# POST /stats/experimenter/<dpid>


class StatsController(ControllerBase):
    def __init__(self, req, link, data, **config):
        super(StatsController, self).__init__(req, link, data, **config)
        self.dpset = data['dpset']
        self.waiters = data['waiters']

    def get_dpids(self, req, **_kwargs):
        dps = list(self.dpset.dps.keys())
        body = json.dumps(dps)
        return Response(content_type='application/json', body=body)

    def get_desc_stats(self, req, dpid, **_kwargs):

        if type(dpid) == str and not dpid.isdigit():
            LOG.debug('invalid dpid %s', dpid)
            return Response(status=400)

        dp = self.dpset.get(int(dpid))

        if dp is None:
            return Response(status=404)
        _ofp_version = dp.ofproto.OFP_VERSION

        _ofctl = supported_ofctl.get(_ofp_version, None)
        if _ofctl is not None:
            desc = _ofctl.get_desc_stats(dp, self.waiters)

        else:
            LOG.debug('Unsupported OF protocol')
            return Response(status=501)

        body = json.dumps(desc)
        return Response(content_type='application/json', body=body)

    def get_flow_stats(self, req, dpid, **_kwargs):

        if req.body == '':
            flow = {}

        else:

            try:
                flow = ast.literal_eval(req.body)

            except SyntaxError:
                LOG.debug('invalid syntax %s', req.body)
                return Response(status=400)

        if type(dpid) == str and not dpid.isdigit():
            LOG.debug('invalid dpid %s', dpid)
            return Response(status=400)

        dp = self.dpset.get(int(dpid))

        if dp is None:
            return Response(status=404)

        _ofp_version = dp.ofproto.OFP_VERSION

        _ofctl = supported_ofctl.get(_ofp_version, None)
        if _ofctl is not None:
            flows = _ofctl.get_flow_stats(dp, self.waiters, flow)

        else:
            LOG.debug('Unsupported OF protocol')
            return Response(status=501)

        body = json.dumps(flows)
        return Response(content_type='application/json', body=body)

    def get_aggregate_flow_stats(self, req, dpid, **_kwargs):

        if req.body == '':
            flow = {}

        else:
            try:
                flow = ast.literal_eval(req.body)

            except SyntaxError:
                LOG.debug('invalid syntax %s', req.body)
                return Response(status=400)

        if type(dpid) == str and not dpid.isdigit():
            LOG.debug('invalid dpid %s', dpid)
            return Response(status=400)

        dp = self.dpset.get(int(dpid))

        if dp is None:
            return Response(status=404)

        _ofp_version = dp.ofproto.OFP_VERSION

        _ofctl = supported_ofctl.get(_ofp_version, None)
        if _ofctl is not None:
            flows = _ofctl.get_aggregate_flow_stats(dp, self.waiters, flow)

        else:
            LOG.debug('Unsupported OF protocol')
            return Response(status=501)

        body = json.dumps(flows)
        return Response(content_type='application/json', body=body)

    def get_table_stats(self, req, dpid, **_kwargs):

        if type(dpid) == str and not dpid.isdigit():
            LOG.debug('invalid dpid %s', dpid)
            return Response(status=400)

        dp = self.dpset.get(int(dpid))

        if dp is None:
            return Response(status=404)

        _ofp_version = dp.ofproto.OFP_VERSION

        _ofctl = supported_ofctl.get(_ofp_version, None)
        if _ofctl is not None:
            ports = _ofctl.get_table_stats(dp, self.waiters)

        else:
            LOG.debug('Unsupported OF protocol')
            return Response(status=501)

        body = json.dumps(ports)
        return Response(content_type='application/json', body=body)

    def get_table_features(self, req, dpid, **_kwargs):

        if type(dpid) == str and not dpid.isdigit():
            LOG.debug('invalid dpid %s', dpid)
            return Response(status=400)

        dp = self.dpset.get(int(dpid))

        if dp is None:
            return Response(status=404)

        _ofp_version = dp.ofproto.OFP_VERSION

        _ofctl = supported_ofctl.get(_ofp_version, None)
        if _ofctl is not None:
            ports = _ofctl.get_table_features(dp, self.waiters)

        else:
            LOG.debug('Unsupported OF protocol')
            return Response(status=501)

        body = json.dumps(ports)
        return Response(content_type='application/json', body=body)

    def get_port_stats(self, req, dpid, **_kwargs):

        if type(dpid) == str and not dpid.isdigit():
            LOG.debug('invalid dpid %s', dpid)
            return Response(status=400)

        dp = self.dpset.get(int(dpid))

        if dp is None:
            return Response(status=404)

        _ofp_version = dp.ofproto.OFP_VERSION

        _ofctl = supported_ofctl.get(_ofp_version, None)
        if _ofctl is not None:
            ports = _ofctl.get_port_stats(dp, self.waiters)

        else:
            LOG.debug('Unsupported OF protocol')
            return Response(status=501)

        body = json.dumps(ports)
        return Response(content_type='application/json', body=body)

    def get_queue_stats(self, req, dpid, **_kwargs):

        if type(dpid) == str and not dpid.isdigit():
            LOG.debug('invalid dpid %s', dpid)
            return Response(status=400)

        dp = self.dpset.get(int(dpid))

        if dp is None:
            return Response(status=404)

        _ofp_version = dp.ofproto.OFP_VERSION

        _ofctl = supported_ofctl.get(_ofp_version, None)
        if _ofctl is not None:
            queues = _ofctl.get_queue_stats(dp, self.waiters)

        else:
            LOG.debug('Unsupported OF protocol')
            return Response(status=501)

        body = json.dumps(queues)
        return Response(content_type='application/json', body=body)

    def get_queue_config(self, req, dpid, port, **_kwargs):

        if type(dpid) == str and not dpid.isdigit():
            LOG.debug('invalid dpid %s', dpid)
            return Response(status=400)

        if type(port) == str and not port.isdigit():
            LOG.debug('invalid port %s', port)
            return Response(status=400)

        dp = self.dpset.get(int(dpid))
        port = int(port)

        if dp is None:
            return Response(status=404)

        _ofp_version = dp.ofproto.OFP_VERSION

        _ofctl = supported_ofctl.get(_ofp_version, None)
        if _ofctl is not None:
            queues = _ofctl.get_queue_config(dp, port, self.waiters)

        else:
            LOG.debug('Unsupported OF protocol')
            return Response(status=501)

        body = json.dumps(queues)
        return Response(content_type='application/json', body=body)

    def get_meter_features(self, req, dpid, **_kwargs):

        if type(dpid) == str and not dpid.isdigit():
            LOG.debug('invalid dpid %s', dpid)
            return Response(status=400)

        dp = self.dpset.get(int(dpid))

        if dp is None:
            return Response(status=404)

        _ofp_version = dp.ofproto.OFP_VERSION
        _ofctl = supported_ofctl.get(_ofp_version, None)

        if _ofctl is not None and hasattr(_ofctl, 'get_meter_features'):
            meters = _ofctl.get_meter_features(dp, self.waiters)

        else:
            LOG.debug('Unsupported OF protocol or \
                request not supported in this OF protocol version')
            return Response(status=501)

        body = json.dumps(meters)
        return Response(content_type='application/json', body=body)

    def get_meter_config(self, req, dpid, **_kwargs):

        if type(dpid) == str and not dpid.isdigit():
            LOG.debug('invalid dpid %s', dpid)
            return Response(status=400)

        dp = self.dpset.get(int(dpid))

        if dp is None:
            return Response(status=404)

        _ofp_version = dp.ofproto.OFP_VERSION
        _ofctl = supported_ofctl.get(_ofp_version, None)

        if _ofctl is not None and hasattr(_ofctl, 'get_meter_config'):
            meters = _ofctl.get_meter_config(dp, self.waiters)

        else:
            LOG.debug('Unsupported OF protocol or \
                request not supported in this OF protocol version')
            return Response(status=501)

        body = json.dumps(meters)
        return Response(content_type='application/json', body=body)

    def get_meter_stats(self, req, dpid, **_kwargs):

        if type(dpid) == str and not dpid.isdigit():
            LOG.debug('invalid dpid %s', dpid)
            return Response(status=400)

        dp = self.dpset.get(int(dpid))

        if dp is None:
            return Response(status=404)

        _ofp_version = dp.ofproto.OFP_VERSION
        _ofctl = supported_ofctl.get(_ofp_version, None)

        if _ofctl is not None and hasattr(_ofctl, 'get_meter_stats'):
            meters = _ofctl.get_meter_stats(dp, self.waiters)

        else:
            LOG.debug('Unsupported OF protocol or \
                request not supported in this OF protocol version')
            return Response(status=501)

        body = json.dumps(meters)
        return Response(content_type='application/json', body=body)

    def get_group_features(self, req, dpid, **_kwargs):

        if type(dpid) == str and not dpid.isdigit():
            LOG.debug('invalid dpid %s', dpid)
            return Response(status=400)

        dp = self.dpset.get(int(dpid))

        if dp is None:
            return Response(status=404)

        _ofp_version = dp.ofproto.OFP_VERSION
        _ofctl = supported_ofctl.get(_ofp_version, None)

        if _ofctl is not None and hasattr(_ofctl, 'get_group_features'):
            groups = _ofctl.get_group_features(dp, self.waiters)

        else:
            LOG.debug('Unsupported OF protocol or \
                request not supported in this OF protocol version')
            return Response(status=501)

        body = json.dumps(groups)
        return Response(content_type='application/json', body=body)

    def get_group_desc(self, req, dpid, **_kwargs):

        if type(dpid) == str and not dpid.isdigit():
            LOG.debug('invalid dpid %s', dpid)
            return Response(status=400)

        dp = self.dpset.get(int(dpid))

        if dp is None:
            return Response(status=404)

        _ofp_version = dp.ofproto.OFP_VERSION
        _ofctl = supported_ofctl.get(_ofp_version, None)

        if _ofctl is not None and hasattr(_ofctl, 'get_group_desc'):
            groups = _ofctl.get_group_desc(dp, self.waiters)

        else:
            LOG.debug('Unsupported OF protocol or \
                request not supported in this OF protocol version')
            return Response(status=501)

        body = json.dumps(groups)
        return Response(content_type='application/json', body=body)

    def get_group_stats(self, req, dpid, **_kwargs):

        if type(dpid) == str and not dpid.isdigit():
            LOG.debug('invalid dpid %s', dpid)
            return Response(status=400)

        dp = self.dpset.get(int(dpid))

        if dp is None:
            return Response(status=404)

        _ofp_version = dp.ofproto.OFP_VERSION
        _ofctl = supported_ofctl.get(_ofp_version, None)

        if _ofctl is not None and hasattr(_ofctl, 'get_group_stats'):
            groups = _ofctl.get_group_stats(dp, self.waiters)

        else:
            LOG.debug('Unsupported OF protocol or \
                request not supported in this OF protocol version')
            return Response(status=501)

        body = json.dumps(groups)
        return Response(content_type='application/json', body=body)

    def get_port_desc(self, req, dpid, **_kwargs):

        if type(dpid) == str and not dpid.isdigit():
            LOG.debug('invalid dpid %s', dpid)
            return Response(status=400)

        dp = self.dpset.get(int(dpid))

        if dp is None:
            return Response(status=404)

        _ofp_version = dp.ofproto.OFP_VERSION

        _ofctl = supported_ofctl.get(_ofp_version, None)
        if _ofctl is not None:
            groups = _ofctl.get_port_desc(dp, self.waiters)

        else:
            LOG.debug('Unsupported OF protocol')
            return Response(status=501)

        body = json.dumps(groups)
        return Response(content_type='application/json', body=body)

    def mod_flow_entry(self, req, cmd, **_kwargs):

        try:
            flow = ast.literal_eval(req.body)

        except SyntaxError:
            LOG.debug('invalid syntax %s', req.body)
            return Response(status=400)

        dpid = flow.get('dpid')

        if type(dpid) == str and not dpid.isdigit():
            LOG.debug('invalid dpid %s', dpid)
            return Response(status=400)

        dp = self.dpset.get(int(dpid))

        if dp is None:
            return Response(status=404)

        if cmd == 'add':
            cmd = dp.ofproto.OFPFC_ADD
        elif cmd == 'modify':
            cmd = dp.ofproto.OFPFC_MODIFY
        elif cmd == 'modify_strict':
            cmd = dp.ofproto.OFPFC_MODIFY_STRICT
        elif cmd == 'delete':
            cmd = dp.ofproto.OFPFC_DELETE
        elif cmd == 'delete_strict':
            cmd = dp.ofproto.OFPFC_DELETE_STRICT
        else:
            return Response(status=404)

        _ofp_version = dp.ofproto.OFP_VERSION
        _ofctl = supported_ofctl.get(_ofp_version, None)
        if _ofctl is not None:
            _ofctl.mod_flow_entry(dp, flow, cmd)
        else:
            LOG.debug('Unsupported OF protocol')
            return Response(status=501)

        return Response(status=200)

    def delete_flow_entry(self, req, dpid, **_kwargs):

        if type(dpid) == str and not dpid.isdigit():
            LOG.debug('invalid dpid %s', dpid)
            return Response(status=400)

        dp = self.dpset.get(int(dpid))

        if dp is None:
            return Response(status=404)

        _ofp_version = dp.ofproto.OFP_VERSION

        if ofproto_v1_0.OFP_VERSION == _ofp_version:
            flow = {}
        else:
            flow = {'table_id': dp.ofproto.OFPTT_ALL}

        _ofctl = supported_ofctl.get(_ofp_version, None)
        if _ofctl is not None:
            _ofctl.mod_flow_entry(dp, flow, dp.ofproto.OFPFC_DELETE)

        else:
            LOG.debug('Unsupported OF protocol')
            return Response(status=501)

        return Response(status=200)

    def mod_meter_entry(self, req, cmd, **_kwargs):

        try:
            flow = ast.literal_eval(req.body)

        except SyntaxError:
            LOG.debug('invalid syntax %s', req.body)
            return Response(status=400)

        dpid = flow.get('dpid')

        if type(dpid) == str and not dpid.isdigit():
            LOG.debug('invalid dpid %s', dpid)
            return Response(status=400)

        dp = self.dpset.get(int(dpid))

        if dp is None:
            return Response(status=404)

        if cmd == 'add':
            cmd = dp.ofproto.OFPMC_ADD
        elif cmd == 'modify':
            cmd = dp.ofproto.OFPMC_MODIFY
        elif cmd == 'delete':
            cmd = dp.ofproto.OFPMC_DELETE
        else:
            return Response(status=404)

        _ofp_version = dp.ofproto.OFP_VERSION
        _ofctl = supported_ofctl.get(_ofp_version, None)

        if _ofctl is not None and hasattr(_ofctl, 'mod_meter_entry'):
            _ofctl.mod_meter_entry(dp, flow, cmd)

        else:
            LOG.debug('Unsupported OF protocol or \
                request not supported in this OF protocol version')
            return Response(status=501)

        return Response(status=200)

    def mod_group_entry(self, req, cmd, **_kwargs):

        try:
            group = ast.literal_eval(req.body)

        except SyntaxError:
            LOG.debug('invalid syntax %s', req.body)
            return Response(status=400)

        dpid = group.get('dpid')

        if type(dpid) == str and not dpid.isdigit():
            LOG.debug('invalid dpid %s', dpid)
            return Response(status=400)

        dp = self.dpset.get(int(dpid))

        if dp is None:
            return Response(status=404)

        if cmd == 'add':
            cmd = dp.ofproto.OFPGC_ADD
        elif cmd == 'modify':
            cmd = dp.ofproto.OFPGC_MODIFY
        elif cmd == 'delete':
            cmd = dp.ofproto.OFPGC_DELETE
        else:
            return Response(status=404)

        _ofp_version = dp.ofproto.OFP_VERSION
        _ofctl = supported_ofctl.get(_ofp_version, None)

        if _ofctl is not None and hasattr(_ofctl, 'mod_group_entry'):
            _ofctl.mod_group_entry(dp, group, cmd)

        else:
            LOG.debug('Unsupported OF protocol or \
                request not supported in this OF protocol version')
            return Response(status=501)

        return Response(status=200)

    def mod_port_behavior(self, req, cmd, **_kwargs):

        try:
            port_config = ast.literal_eval(req.body)

        except SyntaxError:
            LOG.debug('invalid syntax %s', req.body)
            return Response(status=400)

        dpid = port_config.get('dpid')

        if type(dpid) == str and not dpid.isdigit():
            LOG.debug('invalid dpid %s', dpid)
            return Response(status=400)

        port_no = port_config.get('port_no', 0)
        if type(port_no) == str and not port_no.isdigit():
            LOG.debug('invalid port_no %s', port_no)
            return Response(status=400)

        port_info = self.dpset.port_state[int(dpid)].get(port_no)

        if port_info:
            port_config.setdefault('hw_addr', port_info.hw_addr)
            port_config.setdefault('advertise', port_info.advertised)
        else:
            return Response(status=404)

        dp = self.dpset.get(int(dpid))

        if dp is None:
            return Response(status=404)

        if cmd != 'modify':
            return Response(status=404)

        _ofp_version = dp.ofproto.OFP_VERSION

        _ofctl = supported_ofctl.get(_ofp_version, None)
        if _ofctl is not None:
            _ofctl.mod_port_behavior(dp, port_config)

        else:
            LOG.debug('Unsupported OF protocol')
            return Response(status=501)

        return Response(status=200)

    def send_experimenter(self, req, dpid, **_kwargs):

        if type(dpid) == str and not dpid.isdigit():
            LOG.debug('invalid dpid %s', dpid)
            return Response(status=400)

        dp = self.dpset.get(int(dpid))

        if dp is None:
            return Response(status=404)

        try:
            exp = ast.literal_eval(req.body)

        except SyntaxError:
            LOG.debug('invalid syntax %s', req.body)
            return Response(status=400)

        _ofp_version = dp.ofproto.OFP_VERSION
        _ofctl = supported_ofctl.get(_ofp_version, None)

        if _ofctl is not None and hasattr(_ofctl, 'send_experimenter'):
            _ofctl.send_experimenter(dp, exp)

        else:
            LOG.debug('Unsupported OF protocol')
            return Response(status=501)

        return Response(status=200)


class RestStatsApi(app_manager.RyuApp):
    OFP_VERSIONS = [ofproto_v1_0.OFP_VERSION,
                    ofproto_v1_2.OFP_VERSION,
                    ofproto_v1_3.OFP_VERSION]
    _CONTEXTS = {
        'dpset': dpset.DPSet,
        'wsgi': WSGIApplication
    }

    def __init__(self, *args, **kwargs):
        super(RestStatsApi, self).__init__(*args, **kwargs)
        self.dpset = kwargs['dpset']
        wsgi = kwargs['wsgi']
        self.waiters = {}
        self.data = {}
        self.data['dpset'] = self.dpset
        self.data['waiters'] = self.waiters
        mapper = wsgi.mapper

        wsgi.registory['StatsController'] = self.data
        path = '/stats'
        uri = path + '/switches'
        mapper.connect('stats', uri,
                       controller=StatsController, action='get_dpids',
                       conditions=dict(method=['GET']))

        uri = path + '/desc/{dpid}'
        mapper.connect('stats', uri,
                       controller=StatsController, action='get_desc_stats',
                       conditions=dict(method=['GET']))

        uri = path + '/flow/{dpid}'
        mapper.connect('stats', uri,
                       controller=StatsController, action='get_flow_stats',
                       conditions=dict(method=['GET', 'POST']))

        uri = path + '/aggregateflow/{dpid}'
        mapper.connect('stats', uri,
                       controller=StatsController,
                       action='get_aggregate_flow_stats',
                       conditions=dict(method=['GET', 'POST']))

        uri = path + '/table/{dpid}'
        mapper.connect('stats', uri,
                       controller=StatsController, action='get_table_stats',
                       conditions=dict(method=['GET']))

        uri = path + '/tablefeatures/{dpid}'
        mapper.connect('stats', uri,
                       controller=StatsController, action='get_table_features',
                       conditions=dict(method=['GET']))

        uri = path + '/port/{dpid}'
        mapper.connect('stats', uri,
                       controller=StatsController, action='get_port_stats',
                       conditions=dict(method=['GET']))

        uri = path + '/queue/{dpid}'
        mapper.connect('stats', uri,
                       controller=StatsController, action='get_queue_stats',
                       conditions=dict(method=['GET']))

        uri = path + '/queueconfig/{dpid}/{port}'
        mapper.connect('stats', uri,
                       controller=StatsController, action='get_queue_config',
                       conditions=dict(method=['GET']))

        uri = path + '/meterfeatures/{dpid}'
        mapper.connect('stats', uri,
                       controller=StatsController, action='get_meter_features',
                       conditions=dict(method=['GET']))

        uri = path + '/meterconfig/{dpid}'
        mapper.connect('stats', uri,
                       controller=StatsController, action='get_meter_config',
                       conditions=dict(method=['GET']))

        uri = path + '/meter/{dpid}'
        mapper.connect('stats', uri,
                       controller=StatsController, action='get_meter_stats',
                       conditions=dict(method=['GET']))

        uri = path + '/groupfeatures/{dpid}'
        mapper.connect('stats', uri,
                       controller=StatsController, action='get_group_features',
                       conditions=dict(method=['GET']))

        uri = path + '/groupdesc/{dpid}'
        mapper.connect('stats', uri,
                       controller=StatsController, action='get_group_desc',
                       conditions=dict(method=['GET']))

        uri = path + '/group/{dpid}'
        mapper.connect('stats', uri,
                       controller=StatsController, action='get_group_stats',
                       conditions=dict(method=['GET']))

        uri = path + '/portdesc/{dpid}'
        mapper.connect('stats', uri,
                       controller=StatsController, action='get_port_desc',
                       conditions=dict(method=['GET']))

        uri = path + '/flowentry/{cmd}'
        mapper.connect('stats', uri,
                       controller=StatsController, action='mod_flow_entry',
                       conditions=dict(method=['POST']))

        uri = path + '/flowentry/clear/{dpid}'
        mapper.connect('stats', uri,
                       controller=StatsController, action='delete_flow_entry',
                       conditions=dict(method=['DELETE']))

        uri = path + '/meterentry/{cmd}'
        mapper.connect('stats', uri,
                       controller=StatsController, action='mod_meter_entry',
                       conditions=dict(method=['POST']))

        uri = path + '/groupentry/{cmd}'
        mapper.connect('stats', uri,
                       controller=StatsController, action='mod_group_entry',
                       conditions=dict(method=['POST']))

        uri = path + '/portdesc/{cmd}'
        mapper.connect('stats', uri,
                       controller=StatsController, action='mod_port_behavior',
                       conditions=dict(method=['POST']))

        uri = path + '/experimenter/{dpid}'
        mapper.connect('stats', uri,
                       controller=StatsController, action='send_experimenter',
                       conditions=dict(method=['POST']))

    @set_ev_cls([ofp_event.EventOFPStatsReply,
                 ofp_event.EventOFPDescStatsReply,
                 ofp_event.EventOFPFlowStatsReply,
                 ofp_event.EventOFPAggregateStatsReply,
                 ofp_event.EventOFPTableStatsReply,
                 ofp_event.EventOFPTableFeaturesStatsReply,
                 ofp_event.EventOFPPortStatsReply,
                 ofp_event.EventOFPQueueStatsReply,
                 ofp_event.EventOFPMeterStatsReply,
                 ofp_event.EventOFPMeterFeaturesStatsReply,
                 ofp_event.EventOFPMeterConfigStatsReply,
                 ofp_event.EventOFPGroupStatsReply,
                 ofp_event.EventOFPGroupFeaturesStatsReply,
                 ofp_event.EventOFPGroupDescStatsReply,
                 ofp_event.EventOFPPortDescStatsReply
                 ], MAIN_DISPATCHER)
    def stats_reply_handler(self, ev):
        msg = ev.msg
        dp = msg.datapath

        if dp.id not in self.waiters:
            return
        if msg.xid not in self.waiters[dp.id]:
            return
        lock, msgs = self.waiters[dp.id][msg.xid]
        msgs.append(msg)

        flags = 0
        if dp.ofproto.OFP_VERSION == ofproto_v1_0.OFP_VERSION:
            flags = dp.ofproto.OFPSF_REPLY_MORE
        elif dp.ofproto.OFP_VERSION == ofproto_v1_2.OFP_VERSION:
            flags = dp.ofproto.OFPSF_REPLY_MORE
        elif dp.ofproto.OFP_VERSION == ofproto_v1_3.OFP_VERSION:
            flags = dp.ofproto.OFPMPF_REPLY_MORE

        if msg.flags & flags:
            return
        del self.waiters[dp.id][msg.xid]
        lock.set()

    @set_ev_cls([ofp_event.EventOFPSwitchFeatures,
                 ofp_event.EventOFPQueueGetConfigReply], MAIN_DISPATCHER)
    def features_reply_handler(self, ev):
        msg = ev.msg
        dp = msg.datapath

        if dp.id not in self.waiters:
            return
        if msg.xid not in self.waiters[dp.id]:
            return
        lock, msgs = self.waiters[dp.id][msg.xid]
        msgs.append(msg)

        del self.waiters[dp.id][msg.xid]
        lock.set()



# Copyright (C) 2012 Nippon Telegraph and Telephone Corporation.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# vim: tabstop=4 shiftwidth=4 softtabstop=4

import unittest
import logging
import struct
import netaddr
from struct import *
from nose.tools import *

from ryu.lib import mac

LOG = logging.getLogger('test_mac')


class Test_mac(unittest.TestCase):
    """ Test case for mac
    """

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_mac_is_multicast(self):
        addr = b'\x01\x23\x45\x67\x89\x0a'
        val = True

        res = mac.is_multicast(addr)

        eq_(val, res)

    def test_mac_haddr_to_str(self):
        addr = 'aa:aa:aa:aa:aa:aa'
        val = b'\xaa\xaa\xaa\xaa\xaa\xaa'

        res = mac.haddr_to_str(val)

        eq_(addr, res)

    def test_mac_haddr_to_str_none(self):
        """ addr is None
        """
        addr = None
        val = 'None'
        res = mac.haddr_to_str(addr)

        eq_(val, res)

    @raises(AssertionError)
    def test_mac_haddr_to_str_assert(self):
        val = b'\xaa\xaa\xaa\xaa\xaa'

        res = mac.haddr_to_str(val)

    def test_mac_haddr_to_bin_false(self):
        """ len(hexes) = 6 (False)
        """
        addr = 'aa:aa:aa:aa:aa:aa'
        val = b'\xaa\xaa\xaa\xaa\xaa\xaa'

        res = mac.haddr_to_bin(addr)

        eq_(val, res)

    @raises(ValueError)
    def test_mac_haddr_to_bin_true(self):
        """ len(hexes) != 6 (True)
        """
        addr = 'aa:aa:aa:aa:aa'
        res = mac.haddr_to_bin(addr)

    def test_mac_haddr_bitand(self):
        addr = b'\xaa\xaa\xaa\xaa\xaa\xaa'
        mask = b'\xff\xff\xff\x00\x00\x00'
        val = b'\xaa\xaa\xaa\x00\x00\x00'

        res = mac.haddr_bitand(addr, mask)

        eq_(val, res)



# Copyright (C) 2013 Stratosphere Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# vim: tabstop=4 shiftwidth=4 softtabstop=4

import unittest
import logging
from nose.tools import *

from ryu.lib import ofctl_v1_3
from ryu.ofproto import ofproto_v1_3, ofproto_v1_3_parser
from ryu.ofproto import ofproto_protocol
from ryu.ofproto.ofproto_v1_3_parser import OFPActionPopMpls

LOG = logging.getLogger('test_ofctl_v1_3')


class Test_ofctl_v1_3(unittest.TestCase):

    """ Test case for ofctl_v1_3
    """

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_to_actions_pop_mpls(self):
        dp = ofproto_protocol.ProtocolDesc(version=ofproto_v1_3.OFP_VERSION)

        acts = [
            {
                'type': 'POP_MPLS',
                'ethertype': 0x0800
            }
        ]
        result = ofctl_v1_3.to_actions(dp, acts)
        insts = result[0]
        act = insts.actions[0]
        ok_(isinstance(act, OFPActionPopMpls))
        eq_(act.ethertype, 0x0800)



# Copyright (C) 2016 Nippon Telegraph and Telephone Corporation.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging


LOG = logging.getLogger(__name__)


def str_to_int(str_num):
    return int(str(str_num), 0)


class OFCtlUtil(object):

    def __init__(self, ofproto):
        self.ofproto = ofproto

    def _reserved_num_from_user(self, num, prefix):
        if isinstance(num, int):
            return num
        else:
            if num.startswith(prefix):
                return getattr(self.ofproto, num)
            else:
                return getattr(self.ofproto, prefix + num.upper())

    def _reserved_num_to_user(self, num, prefix):
        for k, v in self.ofproto.__dict__.items():
            if k.startswith(prefix) and v == num:
                    return k.replace(prefix, '')
        return num

    def ofp_port_from_user(self, port):
        return self._reserved_num_from_user(port, 'OFPP_')

    def ofp_port_to_user(self, port):
        return self._reserved_num_to_user(port, 'OFPP_')

    def ofp_table_from_user(self, table):
        return self._reserved_num_from_user(table, 'OFPTT_')

    def ofp_table_to_user(self, table):
        return self._reserved_num_to_user(table, 'OFPTT_')

    def ofp_cml_from_user(self, max_len):
        return self._reserved_num_from_user(max_len, 'OFPCML_')

    def ofp_cml_to_user(self, max_len):
        return self._reserved_num_to_user(max_len, 'OFPCML_')

    def ofp_group_from_user(self, group):
        return self._reserved_num_from_user(group, 'OFPG_')

    def ofp_group_to_user(self, group):
        return self._reserved_num_to_user(group, 'OFPG_')

    def ofp_buffer_from_user(self, buffer):
        if buffer in ['OFP_NO_BUFFER', 'NO_BUFFER']:
            return self.ofproto.OFP_NO_BUFFER
        else:
            return buffer

    def ofp_buffer_to_user(self, buffer):
        if self.ofproto.OFP_NO_BUFFER == buffer:
            return 'NO_BUFFER'
        else:
            return buffer

    def ofp_meter_from_user(self, meter):
        return self._reserved_num_from_user(meter, 'OFPM_')

    def ofp_meter_to_user(self, meter):
        return self._reserved_num_to_user(meter, 'OFPM_')

    def ofp_queue_from_user(self, queue):
        return self._reserved_num_from_user(queue, 'OFPQ_')

    def ofp_queue_to_user(self, queue):
        return self._reserved_num_to_user(queue, 'OFPQ_')



# Copyright (C) 2012 Nippon Telegraph and Telephone Corporation.
# Copyright (C) 2012 Isaku Yamahata <yamahata at valinux co jp>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
A dumb OpenFlow 1.0 responder for benchmarking the controller framework.
Intended to be used with oflops cbench.
"""

from ryu.base import app_manager
from ryu.controller import ofp_event
from ryu.controller.handler import MAIN_DISPATCHER
from ryu.controller.handler import set_ev_cls
from ryu.ofproto import ofproto_v1_0


class Cbench(app_manager.RyuApp):
    OFP_VERSIONS = [ofproto_v1_0.OFP_VERSION]

    def __init__(self, *args, **kwargs):
        super(Cbench, self).__init__(*args, **kwargs)

    @set_ev_cls(ofp_event.EventOFPPacketIn, MAIN_DISPATCHER)
    def packet_in_handler(self, ev):
        msg = ev.msg
        datapath = msg.datapath
        ofproto = datapath.ofproto

        match = datapath.ofproto_parser.OFPMatch(
            ofproto_v1_0.OFPFW_ALL, 0, 0, 0,
            0, 0, 0, 0, 0, 0, 0, 0, 0)

        mod = datapath.ofproto_parser.OFPFlowMod(
            datapath, match=match, cookie=0, command=ofproto.OFPFC_ADD,
            idle_timeout=0, hard_timeout=0,
            priority=ofproto.OFP_DEFAULT_PRIORITY,
            flags=0, actions=None)
        datapath.send_msg(mod)



# Copyright (C) 2013-2015 Nippon Telegraph and Telephone Corporation.
# Copyright (C) 2013-2015 YAMAMOTO Takashi <yamamoto at valinux co jp>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import print_function

import base64
import six
import unittest
from nose.tools import eq_

from ryu.lib import stringify


class C1(stringify.StringifyMixin):
    def __init__(self, a, c):
        print("init %s %s" % (a, c))
        self.a = a
        self._b = 'B'
        self.c = c


class Test_stringify(unittest.TestCase):
    """ Test case for ryu.lib.stringify
    """

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_jsondict(self):
        if six.PY3:
            def b64encode(s):
                return base64.b64encode(s).decode('ascii')
        else:
            b64encode = base64.b64encode
        j = {'C1': {'a': 'QUFB', 'c': 'Q0ND'}}
        eq_(j['C1']['a'], b64encode(b'AAA'))
        eq_(j['C1']['c'], b64encode(b'CCC'))
        c = C1(a=b'AAA', c=b'CCC')
        c2 = C1.from_jsondict(j['C1'])
        eq_(c.__class__, c2.__class__)
        eq_(c.__dict__, c2.__dict__)
        eq_(j, c.to_jsondict())

    def test_jsondict2(self):

        def my_encode(x):
            return x.lower()

        def my_decode(x):
            return x.upper()

        j = {'C1': {'a': 'aaa', 'c': 'ccc'}}
        eq_(j['C1']['a'], my_encode('AAA'))
        eq_(j['C1']['c'], my_encode('CCC'))
        c = C1(a='AAA', c='CCC')
        c2 = C1.from_jsondict(j['C1'], decode_string=my_decode)
        eq_(c.__class__, c2.__class__)
        eq_(c.__dict__, c2.__dict__)
        eq_(j, c.to_jsondict(encode_string=my_encode))



# Copyright (C) 2014 Nippon Telegraph and Telephone Corporation.
# Copyright (C) 2014 YAMAMOTO Takashi <yamamoto at valinux co jp>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import unittest
from nose.tools import eq_
from nose.tools import ok_

import sys
import lxml.etree as ET
from formencode.doctest_xml_compare import xml_compare

from ryu.lib.of_config import classes as ofc


GET = """<ns0:capable-switch xmlns:ns0="urn:onf:of111:config:yang">
  <ns0:id>CapableSwitch0</ns0:id>
  <ns0:resources>
    <ns0:port>
      <ns0:resource-id>LogicalSwitch9-Port4</ns0:resource-id>
      <ns0:number>4</ns0:number>
      <ns0:name>Port4</ns0:name>
      <ns0:current-rate>5000</ns0:current-rate>
      <ns0:max-rate>5000</ns0:max-rate>
      <ns0:configuration>
        <ns0:admin-state>up</ns0:admin-state>
        <ns0:no-receive>false</ns0:no-receive>
        <ns0:no-forward>false</ns0:no-forward>
        <ns0:no-packet-in>false</ns0:no-packet-in>
      </ns0:configuration>
      <ns0:state>
        <ns0:oper-state>up</ns0:oper-state>
        <ns0:blocked>false</ns0:blocked>
        <ns0:live>true</ns0:live>
      </ns0:state>
      <ns0:features>
        <ns0:current>
          <ns0:rate>100Mb-FD</ns0:rate>
          <ns0:auto-negotiate>true</ns0:auto-negotiate>
          <ns0:medium>copper</ns0:medium>
          <ns0:pause>unsupported</ns0:pause>
        </ns0:current>
        <ns0:advertised>
          <ns0:rate>other</ns0:rate>
          <ns0:auto-negotiate>true</ns0:auto-negotiate>
          <ns0:medium>copper</ns0:medium>
          <ns0:pause>unsupported</ns0:pause>
        </ns0:advertised>
        <ns0:supported>
          <ns0:rate>100Mb-FD</ns0:rate>
          <ns0:auto-negotiate>true</ns0:auto-negotiate>
          <ns0:medium>copper</ns0:medium>
          <ns0:pause>unsupported</ns0:pause>
        </ns0:supported>
        <ns0:advertised-peer>
          <ns0:rate>100Mb-FD</ns0:rate>
          <ns0:auto-negotiate>true</ns0:auto-negotiate>
          <ns0:medium>copper</ns0:medium>
          <ns0:pause>unsupported</ns0:pause>
        </ns0:advertised-peer>
      </ns0:features>
    </ns0:port>
    <ns0:port>
      <ns0:resource-id>LogicalSwitch9-Port3</ns0:resource-id>
      <ns0:number>3</ns0:number>
      <ns0:name>Port3</ns0:name>
      <ns0:current-rate>5000</ns0:current-rate>
      <ns0:max-rate>5000</ns0:max-rate>
      <ns0:configuration>
        <ns0:admin-state>up</ns0:admin-state>
        <ns0:no-receive>false</ns0:no-receive>
        <ns0:no-forward>false</ns0:no-forward>
        <ns0:no-packet-in>false</ns0:no-packet-in>
      </ns0:configuration>
      <ns0:state>
        <ns0:oper-state>up</ns0:oper-state>
        <ns0:blocked>false</ns0:blocked>
        <ns0:live>true</ns0:live>
      </ns0:state>
      <ns0:features>
        <ns0:current>
          <ns0:rate>100Mb-FD</ns0:rate>
          <ns0:auto-negotiate>true</ns0:auto-negotiate>
          <ns0:medium>copper</ns0:medium>
          <ns0:pause>unsupported</ns0:pause>
        </ns0:current>
        <ns0:advertised>
          <ns0:rate>other</ns0:rate>
          <ns0:auto-negotiate>true</ns0:auto-negotiate>
          <ns0:medium>copper</ns0:medium>
          <ns0:pause>unsupported</ns0:pause>
        </ns0:advertised>
        <ns0:supported>
          <ns0:rate>100Mb-FD</ns0:rate>
          <ns0:auto-negotiate>true</ns0:auto-negotiate>
          <ns0:medium>copper</ns0:medium>
          <ns0:pause>unsupported</ns0:pause>
        </ns0:supported>
        <ns0:advertised-peer>
          <ns0:rate>100Mb-FD</ns0:rate>
          <ns0:auto-negotiate>true</ns0:auto-negotiate>
          <ns0:medium>copper</ns0:medium>
          <ns0:pause>unsupported</ns0:pause>
        </ns0:advertised-peer>
      </ns0:features>
    </ns0:port>
    <ns0:port>
      <ns0:resource-id>LogicalSwitch7-Port2</ns0:resource-id>
      <ns0:number>2</ns0:number>
      <ns0:name>Port2</ns0:name>
      <ns0:current-rate>5000</ns0:current-rate>
      <ns0:max-rate>5000</ns0:max-rate>
      <ns0:configuration>
        <ns0:admin-state>up</ns0:admin-state>
        <ns0:no-receive>false</ns0:no-receive>
        <ns0:no-forward>false</ns0:no-forward>
        <ns0:no-packet-in>false</ns0:no-packet-in>
      </ns0:configuration>
      <ns0:state>
        <ns0:oper-state>up</ns0:oper-state>
        <ns0:blocked>false</ns0:blocked>
        <ns0:live>true</ns0:live>
      </ns0:state>
      <ns0:features>
        <ns0:current>
          <ns0:rate>100Mb-FD</ns0:rate>
          <ns0:auto-negotiate>true</ns0:auto-negotiate>
          <ns0:medium>copper</ns0:medium>
          <ns0:pause>unsupported</ns0:pause>
        </ns0:current>
        <ns0:advertised>
          <ns0:rate>other</ns0:rate>
          <ns0:auto-negotiate>true</ns0:auto-negotiate>
          <ns0:medium>copper</ns0:medium>
          <ns0:pause>unsupported</ns0:pause>
        </ns0:advertised>
        <ns0:supported>
          <ns0:rate>100Mb-FD</ns0:rate>
          <ns0:auto-negotiate>true</ns0:auto-negotiate>
          <ns0:medium>copper</ns0:medium>
          <ns0:pause>unsupported</ns0:pause>
        </ns0:supported>
        <ns0:advertised-peer>
          <ns0:rate>100Mb-FD</ns0:rate>
          <ns0:auto-negotiate>true</ns0:auto-negotiate>
          <ns0:medium>copper</ns0:medium>
          <ns0:pause>unsupported</ns0:pause>
        </ns0:advertised-peer>
      </ns0:features>
    </ns0:port>
    <ns0:port>
      <ns0:resource-id>LogicalSwitch7-Port1</ns0:resource-id>
      <ns0:number>1</ns0:number>
      <ns0:name>Port1</ns0:name>
      <ns0:current-rate>5000</ns0:current-rate>
      <ns0:max-rate>5000</ns0:max-rate>
      <ns0:configuration>
        <ns0:admin-state>up</ns0:admin-state>
        <ns0:no-receive>false</ns0:no-receive>
        <ns0:no-forward>false</ns0:no-forward>
        <ns0:no-packet-in>false</ns0:no-packet-in>
      </ns0:configuration>
      <ns0:state>
        <ns0:oper-state>up</ns0:oper-state>
        <ns0:blocked>false</ns0:blocked>
        <ns0:live>true</ns0:live>
      </ns0:state>
      <ns0:features>
        <ns0:current>
          <ns0:rate>100Mb-FD</ns0:rate>
          <ns0:auto-negotiate>true</ns0:auto-negotiate>
          <ns0:medium>copper</ns0:medium>
          <ns0:pause>unsupported</ns0:pause>
        </ns0:current>
        <ns0:advertised>
          <ns0:rate>other</ns0:rate>
          <ns0:auto-negotiate>true</ns0:auto-negotiate>
          <ns0:medium>copper</ns0:medium>
          <ns0:pause>unsupported</ns0:pause>
        </ns0:advertised>
        <ns0:supported>
          <ns0:rate>100Mb-FD</ns0:rate>
          <ns0:auto-negotiate>true</ns0:auto-negotiate>
          <ns0:medium>copper</ns0:medium>
          <ns0:pause>unsupported</ns0:pause>
        </ns0:supported>
        <ns0:advertised-peer>
          <ns0:rate>100Mb-FD</ns0:rate>
          <ns0:auto-negotiate>true</ns0:auto-negotiate>
          <ns0:medium>copper</ns0:medium>
          <ns0:pause>unsupported</ns0:pause>
        </ns0:advertised-peer>
      </ns0:features>
    </ns0:port>
    <ns0:queue>
      <ns0:resource-id>LogicalSwitch9-Port4-Queue992</ns0:resource-id>
      <ns0:id>992</ns0:id>
      <ns0:port>4</ns0:port>
      <ns0:properties>
        <ns0:min-rate>10</ns0:min-rate>
        <ns0:max-rate>130</ns0:max-rate>
      </ns0:properties>
    </ns0:queue>
    <ns0:queue>
      <ns0:resource-id>LogicalSwitch9-Port4-Queue991</ns0:resource-id>
      <ns0:id>991</ns0:id>
      <ns0:port>4</ns0:port>
      <ns0:properties>
        <ns0:min-rate>10</ns0:min-rate>
        <ns0:max-rate>120</ns0:max-rate>
      </ns0:properties>
    </ns0:queue>
    <ns0:queue>
      <ns0:resource-id>LogicalSwitch7-Port2-Queue994</ns0:resource-id>
      <ns0:id>994</ns0:id>
      <ns0:port>2</ns0:port>
      <ns0:properties>
        <ns0:min-rate>400</ns0:min-rate>
        <ns0:max-rate>900</ns0:max-rate>
      </ns0:properties>
    </ns0:queue>
    <ns0:queue>
      <ns0:resource-id>LogicalSwitch7-Port2-Queue993</ns0:resource-id>
      <ns0:id>993</ns0:id>
      <ns0:port>2</ns0:port>
      <ns0:properties>
        <ns0:min-rate>200</ns0:min-rate>
        <ns0:max-rate>300</ns0:max-rate>
      </ns0:properties>
    </ns0:queue>
  </ns0:resources>
  <ns0:logical-switches>
    <ns0:switch>
      <ns0:id>LogicalSwitch9</ns0:id>
      <ns0:capabilities>
        <ns0:max-buffered-packets>0</ns0:max-buffered-packets>
        <ns0:max-tables>255</ns0:max-tables>
        <ns0:max-ports>16777216</ns0:max-ports>
        <ns0:flow-statistics>true</ns0:flow-statistics>
        <ns0:table-statistics>true</ns0:table-statistics>
        <ns0:port-statistics>true</ns0:port-statistics>
        <ns0:group-statistics>true</ns0:group-statistics>
        <ns0:queue-statistics>true</ns0:queue-statistics>
        <ns0:reassemble-ip-fragments>false</ns0:reassemble-ip-fragments>
        <ns0:block-looping-ports>false</ns0:block-looping-ports>
        <ns0:reserved-port-types>
          <ns0:type>all</ns0:type>
          <ns0:type>controller</ns0:type>
          <ns0:type>table</ns0:type>
          <ns0:type>inport</ns0:type>
          <ns0:type>any</ns0:type>
        </ns0:reserved-port-types>
        <ns0:group-types>
          <ns0:type>all</ns0:type>
          <ns0:type>select</ns0:type>
          <ns0:type>indirect</ns0:type>
          <ns0:type>fast-failover</ns0:type>
        </ns0:group-types>
        <ns0:group-capabilities>
          <ns0:capability>select-weight</ns0:capability>
          <ns0:capability>select-liveness</ns0:capability>
          <ns0:capability>chaining</ns0:capability>
        </ns0:group-capabilities>
        <ns0:action-types>
          <ns0:type>output</ns0:type>
          <ns0:type>group</ns0:type>
          <ns0:type>set-queue</ns0:type>
          <ns0:type>set-mpls-ttl</ns0:type>
          <ns0:type>dec-mpls-ttl</ns0:type>
          <ns0:type>set-nw-ttl</ns0:type>
          <ns0:type>dec-nw-ttl</ns0:type>
          <ns0:type>copy-ttl-out</ns0:type>
          <ns0:type>copy-ttl-in</ns0:type>
          <ns0:type>push-vlan</ns0:type>
          <ns0:type>pop-vlan</ns0:type>
          <ns0:type>push-mpls</ns0:type>
          <ns0:type>pop-mpls</ns0:type>
          <ns0:type>push-pbb</ns0:type>
          <ns0:type>pop-pbb</ns0:type>
          <ns0:type>set-field</ns0:type>
        </ns0:action-types>
        <ns0:instruction-types>
          <ns0:type>goto-table</ns0:type>
          <ns0:type>write-metadata</ns0:type>
          <ns0:type>write-actions</ns0:type>
          <ns0:type>apply-actions</ns0:type>
          <ns0:type>clear-actions</ns0:type>
          <ns0:type>meter</ns0:type>
        </ns0:instruction-types>
      </ns0:capabilities>
      <ns0:datapath-id>08:60:6E:7F:74:E7:00:09</ns0:datapath-id>
      <ns0:enabled>true</ns0:enabled>
      <ns0:check-controller-certificate>false
        </ns0:check-controller-certificate>
      <ns0:lost-connection-behavior>failSecureMode
        </ns0:lost-connection-behavior>
      <ns0:controllers>
        <ns0:controller>
          <ns0:id>Switch9-Controller</ns0:id>
          <ns0:role>equal</ns0:role>
          <ns0:ip-address>127.0.0.1</ns0:ip-address>
          <ns0:port>6633</ns0:port>
          <ns0:protocol>tcp</ns0:protocol>
          <ns0:state>
            <ns0:connection-state>down</ns0:connection-state>
            <ns0:supported-versions>1.3</ns0:supported-versions>
          </ns0:state>
        </ns0:controller>
      </ns0:controllers>
      <ns0:resources>
        <ns0:port>LogicalSwitch9-Port4</ns0:port>
        <ns0:port>LogicalSwitch9-Port3</ns0:port>
        <ns0:queue>LogicalSwitch9-Port4-Queue992</ns0:queue>
        <ns0:queue>LogicalSwitch9-Port4-Queue991</ns0:queue>
      </ns0:resources>
    </ns0:switch>
    <ns0:switch>
      <ns0:id>LogicalSwitch7</ns0:id>
      <ns0:capabilities>
        <ns0:max-buffered-packets>0</ns0:max-buffered-packets>
        <ns0:max-tables>255</ns0:max-tables>
        <ns0:max-ports>16777216</ns0:max-ports>
        <ns0:flow-statistics>true</ns0:flow-statistics>
        <ns0:table-statistics>true</ns0:table-statistics>
        <ns0:port-statistics>true</ns0:port-statistics>
        <ns0:group-statistics>true</ns0:group-statistics>
        <ns0:queue-statistics>true</ns0:queue-statistics>
        <ns0:reassemble-ip-fragments>false</ns0:reassemble-ip-fragments>
        <ns0:block-looping-ports>false</ns0:block-looping-ports>
        <ns0:reserved-port-types>
          <ns0:type>all</ns0:type>
          <ns0:type>controller</ns0:type>
          <ns0:type>table</ns0:type>
          <ns0:type>inport</ns0:type>
          <ns0:type>any</ns0:type>
        </ns0:reserved-port-types>
        <ns0:group-types>
          <ns0:type>all</ns0:type>
          <ns0:type>select</ns0:type>
          <ns0:type>indirect</ns0:type>
          <ns0:type>fast-failover</ns0:type>
        </ns0:group-types>
        <ns0:group-capabilities>
          <ns0:capability>select-weight</ns0:capability>
          <ns0:capability>select-liveness</ns0:capability>
          <ns0:capability>chaining</ns0:capability>
        </ns0:group-capabilities>
        <ns0:action-types>
          <ns0:type>output</ns0:type>
          <ns0:type>group</ns0:type>
          <ns0:type>set-queue</ns0:type>
          <ns0:type>set-mpls-ttl</ns0:type>
          <ns0:type>dec-mpls-ttl</ns0:type>
          <ns0:type>set-nw-ttl</ns0:type>
          <ns0:type>dec-nw-ttl</ns0:type>
          <ns0:type>copy-ttl-out</ns0:type>
          <ns0:type>copy-ttl-in</ns0:type>
          <ns0:type>push-vlan</ns0:type>
          <ns0:type>pop-vlan</ns0:type>
          <ns0:type>push-mpls</ns0:type>
          <ns0:type>pop-mpls</ns0:type>
          <ns0:type>set-field</ns0:type>
        </ns0:action-types>
        <ns0:instruction-types>
          <ns0:type>goto-table</ns0:type>
          <ns0:type>write-metadata</ns0:type>
          <ns0:type>write-actions</ns0:type>
          <ns0:type>apply-actions</ns0:type>
          <ns0:type>clear-actions</ns0:type>
        </ns0:instruction-types>
      </ns0:capabilities>
      <ns0:datapath-id>08:60:6E:7F:74:E7:00:07</ns0:datapath-id>
      <ns0:enabled>true</ns0:enabled>
      <ns0:check-controller-certificate>false
        </ns0:check-controller-certificate>
      <ns0:lost-connection-behavior>failSecureMode
        </ns0:lost-connection-behavior>
      <ns0:controllers>
        <ns0:controller>
          <ns0:id>Switch7-Controller</ns0:id>
          <ns0:role>equal</ns0:role>
          <ns0:ip-address>127.0.0.1</ns0:ip-address>
          <ns0:port>6633</ns0:port>
          <ns0:protocol>tcp</ns0:protocol>
          <ns0:state>
            <ns0:connection-state>down</ns0:connection-state>
            <ns0:supported-versions>1.2</ns0:supported-versions>
          </ns0:state>
        </ns0:controller>
      </ns0:controllers>
      <ns0:resources>
        <ns0:port>LogicalSwitch7-Port2</ns0:port>
        <ns0:port>LogicalSwitch7-Port1</ns0:port>
        <ns0:queue>LogicalSwitch7-Port2-Queue994</ns0:queue>
        <ns0:queue>LogicalSwitch7-Port2-Queue993</ns0:queue>
      </ns0:resources>
    </ns0:switch>
  </ns0:logical-switches>
</ns0:capable-switch>
"""


class Test_of_config_classes(unittest.TestCase):
    """ Test case for ryu.lib.of_config.classes
    """

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_parse(self):
        for xml0 in [GET]:
            o = ofc.OFCapableSwitchType.from_xml(xml0)
            xml1 = o.to_xml('capable-switch')
            ok_(xml_compare(ET.fromstring(xml0), ET.fromstring(xml1),
                            reporter=sys.stderr.write))

    def test_alt_names(self):
        xml0 = GET
        o = ofc.OFCapableSwitchType.from_xml(xml0)
        eq_(o.logical_switches, getattr(o, 'logical_switches'))
        eq_(o.logical_switches, getattr(o, 'logical-switches'))

    def test_iterate(self):
        xml0 = GET
        o = ofc.OFCapableSwitchType.from_xml(xml0)
        for lsw in o.logical_switches.switch:
            ok_(str(lsw.id).startswith('LogicalSwitch'))



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from Queue import Queue
import struct
import threading

from SocketServer import TCPServer, BaseRequestHandler, ThreadingMixIn

from . import RPCRequestResponseServer


def _read_length_prefixed_msg(sock, prefix_format='!I'):
    prefix_bytes = struct.calcsize(prefix_format)

    sock.recv(prefix_bytes)

def _read_n_bytes(sock, n):
    buf = []
    while n > 0:
        data = sock.recv(n)
        n -= len(data)
        buf.append(data)

    return ''.join(buf)


def create_length_prefixed_tcp_handler():
    queue = Queue()
    class LengthPrefixedTcpHandler(BaseRequestHandler):
        def handle(self):
            #msg = _read_length_prefixed_msg(self.request)
            # this will run inside a new thread
            self.request.send("hello\n")
            while True:
                b = _read_n_bytes(self.request, 10)
                self.request.send("you sent: %s" % b)
                queue.put(b)

    return queue, LengthPrefixedTcpHandler


def tcp_test_main():
    class Server(ThreadingMixIn, TCPServer):
        pass

    queue, Handler = create_length_prefixed_tcp_handler()

    server = Server(('localhost', 12345), Handler)
    server.allow_reuse_address = True

    server.serve_forever()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import absolute_import
import gevent

from . import RPCServer


class RPCServerGreenlets(RPCServer):
    # documentation in docs because of dependencies
    def _spawn(self, func, *args, **kwargs):
        gevent.spawn(func, *args, **kwargs)



import logging
import netaddr
from collections import OrderedDict

from ryu.services.protocols.bgp.base import SUPPORTED_GLOBAL_RF
from ryu.services.protocols.bgp.info_base.rtc import RtcTable
from ryu.services.protocols.bgp.info_base.ipv4 import Ipv4Path
from ryu.services.protocols.bgp.info_base.ipv4 import Ipv4Table
from ryu.services.protocols.bgp.info_base.ipv6 import Ipv6Path
from ryu.services.protocols.bgp.info_base.ipv6 import Ipv6Table
from ryu.services.protocols.bgp.info_base.vpnv4 import Vpnv4Path
from ryu.services.protocols.bgp.info_base.vpnv4 import Vpnv4Table
from ryu.services.protocols.bgp.info_base.vpnv6 import Vpnv6Path
from ryu.services.protocols.bgp.info_base.vpnv6 import Vpnv6Table
from ryu.services.protocols.bgp.info_base.vrf4 import Vrf4Table
from ryu.services.protocols.bgp.info_base.vrf6 import Vrf6Table
from ryu.services.protocols.bgp.rtconf import vrfs
from ryu.services.protocols.bgp.rtconf.vrfs import VRF_RF_IPV4
from ryu.services.protocols.bgp.rtconf.vrfs import VRF_RF_IPV6

from ryu.lib.packet.bgp import RF_IPv4_UC
from ryu.lib.packet.bgp import RF_IPv6_UC
from ryu.lib.packet.bgp import RF_IPv4_VPN
from ryu.lib.packet.bgp import RF_IPv6_VPN
from ryu.lib.packet.bgp import RF_RTC_UC
from ryu.lib.packet.bgp import BGPPathAttributeOrigin
from ryu.lib.packet.bgp import BGPPathAttributeAsPath
from ryu.lib.packet.bgp import BGP_ATTR_TYPE_ORIGIN
from ryu.lib.packet.bgp import BGP_ATTR_TYPE_AS_PATH
from ryu.lib.packet.bgp import BGP_ATTR_ORIGIN_IGP
from ryu.lib.packet.bgp import IPAddrPrefix
from ryu.lib.packet.bgp import IP6AddrPrefix

from ryu.services.protocols.bgp.utils.validation import is_valid_ipv4
from ryu.services.protocols.bgp.utils.validation import is_valid_ipv4_prefix
from ryu.services.protocols.bgp.utils.validation import is_valid_ipv6
from ryu.services.protocols.bgp.utils.validation import is_valid_ipv6_prefix


LOG = logging.getLogger('bgpspeaker.core_managers.table_mixin')


class TableCoreManager(object):
    """Methods performing core operations on tables."""

    def __init__(self, core_service, common_conf):

        self._tables = {}
        self._rt_mgr = core_service.rt_manager
        self._signal_bus = core_service.signal_bus

        # (VRF) Tables to which the routes with a given route target
        # should be imported.
        #
        # Key: RouteTarget
        # Value: List of tables.
        self._tables_for_rt = {}

        # Global/Default tables, keyed by RouteFamily.
        self._global_tables = {}

        self._core_service = core_service
        self._signal_bus = self._core_service.signal_bus

        # VPN label range
        self._asbr_label_range = common_conf.label_range

        self._next_vpnv4_label = int(self._asbr_label_range[0])

        self._next_hop_label = {}

    @property
    def global_tables(self):
        return self._global_tables

    def remove_vrf_by_vrf_conf(self, vrf_conf):

        route_family = vrf_conf.route_family
        assert route_family in (vrfs.VRF_RF_IPV4, vrfs.VRF_RF_IPV6)
        table_id = (vrf_conf.route_dist, route_family)

        vrf_table = self._tables.pop(table_id)

        self._remove_links_to_vrf_table(vrf_table)

        # Withdraw the best-path whose source was NC since it may have been
        # exported to VPN table.
        for destination in vrf_table.values():
            best_path = destination.best_path
            if best_path and best_path.source is None:
                vpn_clone = best_path.clone_to_vpn(vrf_conf.route_dist,
                                                   for_withdrawal=True)
                self.learn_path(vpn_clone)
        LOG.debug('VRF with RD %s marked for removal', vrf_conf.route_dist)

    def import_all_vpn_paths_to_vrf(self, vrf_table, import_rts=None):
        """Imports Vpnv4/6 paths from Global/VPN table into given Vrfv4/6
         table.
        :param vrf_table: Vrf table to which we import
        :type vrf_table: VrfTable
        :param import_rts: import RTs to override default import_rts of
         vrf table for this import
        :type import_rts: set of strings


        Checks if we have any path RT common with VRF table's import RT.
        """
        rfs = (Vrf4Table.ROUTE_FAMILY, Vrf6Table.ROUTE_FAMILY)
        assert vrf_table.route_family in rfs, 'Invalid VRF table.'

        if vrf_table.route_family == Vrf4Table.ROUTE_FAMILY:
            vpn_table = self.get_vpn4_table()
        else:
            vpn_table = self.get_vpn6_table()

        vrf_table.import_vpn_paths_from_table(vpn_table, import_rts)

    def learn_path(self, path):
        """Inserts `path` into correct global table.

        Since known paths to `Destination` has changes, we queue it for further
        processing.
        """
        # Get VPN/Global table
        table = self.get_global_table_by_route_family(path.route_family)
        gpath_dest = table.insert(path)
        # Since destination was updated, we enqueue it for processing.
        self._signal_bus.dest_changed(gpath_dest)

    def remember_sent_route(self, sent_route):
        """Records `sent_route` inside proper table.

        Records of `sent_route` from Adj-RIB-out.
        """
        route_family = sent_route.path.route_family
        table = self.get_global_table_by_route_family(route_family)
        table.insert_sent_route(sent_route)

    def on_interesting_rts_change(self, new_global_rts, removed_global_rts):
        """Update global tables as interested RTs changed.

        Adds `new_rts` and removes `removed_rts` rt nlris. Does not check if
        `new_rts` or `removed_rts` are already present. Schedules refresh
        request to peers that do not participate in RTC address-family.
        """
        # We add new RT NLRI and request RR for other peers.
        if new_global_rts:
            LOG.debug(
                'Sending route_refresh to all neighbors that'
                ' did not negotiate RTC capability.'
            )

            pm = self._core_service.peer_manager
            pm.schedule_rr_to_non_rtc_peers()
        if removed_global_rts:
            LOG.debug(
                'Cleaning up global tables as some interested RTs were removed'
            )
            self._clean_global_uninteresting_paths()

    def get_global_table_by_route_family(self, route_family):
        if route_family not in SUPPORTED_GLOBAL_RF:
            raise ValueError(
                'Given route family: %s currently not supported' % route_family
            )

        global_table = None
        if route_family == RF_IPv4_UC:
            global_table = self.get_ipv4_table()
        elif route_family == RF_IPv6_UC:
            global_table = self.get_ipv6_table()
        elif route_family == RF_IPv4_VPN:
            global_table = self.get_vpn4_table()

        elif route_family == RF_IPv6_VPN:
            global_table = self.get_vpn6_table()

        elif route_family == RF_RTC_UC:
            global_table = self.get_rtc_table()

        return global_table

    def get_vrf_table(self, vrf_rd, vrf_rf):
        assert vrf_rd is not None
        return self._tables.get((vrf_rd, vrf_rf))

    def get_vrf_tables(self, vrf_rf=None):
        vrf_tables = {}
        for (scope_id, table_id), table in self._tables.items():
            if scope_id is None:
                continue
            if vrf_rf is not None and table_id != vrf_rf:
                continue
            vrf_tables[(scope_id, table_id)] = table
        return vrf_tables

    def get_ipv4_table(self):
        """Returns global IPv4 table.

        Creates the table if it does not exist.
        """

        vpn_table = self._global_tables.get(RF_IPv4_UC)
        # Lazy initialize the table.
        if not vpn_table:
            vpn_table = Ipv4Table(self._core_service, self._signal_bus)
            self._global_tables[RF_IPv4_UC] = vpn_table
            self._tables[(None, RF_IPv4_UC)] = vpn_table

        return vpn_table

    def get_ipv6_table(self):
        table = self._global_tables.get(RF_IPv6_UC)
        if not table:
            table = Ipv6Table(self._core_service, self._signal_bus)
            self._global_tables[RF_IPv6_UC] = table
            self._tables[(None, RF_IPv6_UC)] = table
        return table

    def get_vpn6_table(self):
        """Returns global VPNv6 table.

        Creates the table if it does not exist.
        """
        vpn_table = self._global_tables.get(RF_IPv6_VPN)
        # Lazy initialize the table.
        if not vpn_table:
            vpn_table = Vpnv6Table(self._core_service, self._signal_bus)
            self._global_tables[RF_IPv6_VPN] = vpn_table
            self._tables[(None, RF_IPv6_VPN)] = vpn_table

        return vpn_table

    def get_vpn4_table(self):
        """Returns global VPNv6 table.

        Creates the table if it does not exist.
        """
        vpn_table = self._global_tables.get(RF_IPv4_VPN)
        # Lazy initialize the table.
        if not vpn_table:
            vpn_table = Vpnv4Table(self._core_service, self._signal_bus)
            self._global_tables[RF_IPv4_VPN] = vpn_table
            self._tables[(None, RF_IPv4_VPN)] = vpn_table

        return vpn_table

    def get_rtc_table(self):
        """Returns global RTC table.

        Creates the table if it does not exist.
        """
        rtc_table = self._global_tables.get(RF_RTC_UC)
        # Lazy initialization of the table.
        if not rtc_table:
            rtc_table = RtcTable(self._core_service, self._signal_bus)
            self._global_tables[RF_RTC_UC] = rtc_table
            self._tables[(None, RF_RTC_UC)] = rtc_table
        return rtc_table

    def get_next_vpnv4_label(self):
        # Get next available label
        lbl = self._next_vpnv4_label
        # Check if label is within max. range allowed.
        if lbl > int(self._asbr_label_range[1]):
            # Currently we log error message if we exceed configured range.
            message = 'Have reached max label range'
            LOG.error(message)
            raise ValueError(message)
            # Increment label by 1 as next label.
        self._next_vpnv4_label += 1
        return lbl

    def get_nexthop_label(self, label_key):
        return self._next_hop_label.get(label_key, None)

    def set_nexthop_label(self, key, value):
        self._next_hop_label[key] = value

    def update_vrf_table_links(self, vrf_table, new_imp_rts,
                               removed_imp_rts):
        """Update mapping from RT to VRF table."""
        assert vrf_table
        if new_imp_rts:
            self._link_vrf_table(vrf_table, new_imp_rts)
        if removed_imp_rts:
            self._remove_links_to_vrf_table_for_rts(vrf_table,
                                                    removed_imp_rts)

    def re_install_net_ctrl_paths(self, vrf_table):
        """Re-installs paths from NC with current BGP policy.

        Iterates over known paths from NC installed in `vrf4_table` and
        adds new path with path attributes as per current VRF configuration.
        """
        assert vrf_table
        for dest in vrf_table.values():
            for path in dest.known_path_list:
                if path.source is None:
                    vrf_table.insert_vrf_path(
                        path.nlri,
                        path.nexthop,
                        gen_lbl=True
                    )
        LOG.debug('Re-installed NC paths with current policy for table %s.',
                  vrf_table)

    def _remove_links_to_vrf_table(self, vrf_table):
        """Removes any links to given `vrf_table`."""
        assert vrf_table
        vrf_conf = vrf_table.vrf_conf
        self._remove_links_to_vrf_table_for_rts(vrf_table,
                                                vrf_conf.import_rts)

    def _remove_links_to_vrf_table_for_rts(self, vrf_table, rts):
        rts_with_no_table = set()
        affected_tables = set()
        route_family = vrf_table.route_family
        for rt in rts:
            rt_rf_id = rt + ':' + str(route_family)
            rt_specific_tables = self._tables_for_rt.get(rt_rf_id)
            affected_tables.update(rt_specific_tables)
            if rt_specific_tables:
                try:
                    rt_specific_tables.remove(vrf_table)
                except KeyError:
                    LOG.debug('Did not find table listed as interested '
                              'for its import RT: %s', rt)
                if len(rt_specific_tables) == 0:
                    rts_with_no_table.add(rt)

        # Remove records of RT that have no tables associated with it.
        for rt in rts_with_no_table:
            rt_rf_id = rt + ':' + str(route_family)
            del self._tables_for_rt[rt_rf_id]

    def create_and_link_vrf_table(self, vrf_conf):
        """Factory method to create VRF table for given `vrf_conf`.

        Adds mapping to this table with appropriate scope. Also, adds mapping
        for import RT of this VRF to created table to facilitate
        importing/installing of paths from global tables.
        Returns created table.
        """

        route_family = vrf_conf.route_family
        assert route_family in (VRF_RF_IPV4, VRF_RF_IPV6)
        vrf_table = None
        if route_family == VRF_RF_IPV4:
            vrf_table = Vrf4Table(
                vrf_conf, self._core_service, self._signal_bus
            )
            table_id = (vrf_conf.route_dist, route_family)
            self._tables[table_id] = vrf_table

        elif route_family == VRF_RF_IPV6:
            vrf_table = Vrf6Table(
                vrf_conf, self._core_service, self._signal_bus
            )
            table_id = (vrf_conf.route_dist, route_family)
            self._tables[table_id] = vrf_table

        assert vrf_table is not None
        LOG.debug('Added new VrfTable with rd: %s and add_fmly: %s',
                  vrf_conf.route_dist, route_family)

        import_rts = vrf_conf.import_rts
        # If VRF is configured with import RT, we put this table
        # in a list corresponding to this RT for easy access.
        if import_rts:
            self._link_vrf_table(vrf_table, import_rts)

        return vrf_table

    def _link_vrf_table(self, vrf_table, rt_list):
        route_family = vrf_table.route_family
        for rt in rt_list:
            rt_rf_id = rt + ':' + str(route_family)
            table_set = self._tables_for_rt.get(rt_rf_id)
            if table_set is None:
                table_set = set()
                self._tables_for_rt[rt_rf_id] = table_set
            table_set.add(vrf_table)
            LOG.debug('Added VrfTable %s to import RT table list: %s',
                      vrf_table, rt)

    def _clean_global_uninteresting_paths(self):
        """Marks paths that do not have any route targets of interest
        for withdrawal.

        Since global tables can have paths with route targets that are not
        interesting any more, we have to clean these paths so that appropriate
        withdraw are sent out to NC and other peers. Interesting route targets
        change as VRF are modified or some filter is that specify what route
        targets are allowed are updated. This clean up should only be done when
        a route target is no longer considered interesting and some paths with
        that route target was installing in any of the global table.
        """
        uninteresting_dest_count = 0
        interested_rts = self._rt_mgr.global_interested_rts
        LOG.debug('Cleaning uninteresting paths. Global interested RTs %s',
                  interested_rts)
        for route_family in [RF_IPv4_VPN, RF_IPv6_VPN, RF_RTC_UC]:
            # TODO(PH): We currently do not install RT_NLRI paths based on
            # extended path attributes (RT)
            if route_family == RF_RTC_UC:
                continue
            table = self.get_global_table_by_route_family(route_family)
            uninteresting_dest_count += \
                table.clean_uninteresting_paths(interested_rts)

        LOG.debug('Found %s number of destinations had uninteresting paths.',
                  uninteresting_dest_count)

    def import_single_vpn_path_to_all_vrfs(self, vpn_path, path_rts=None):
        """Imports *vpnv4_path* to qualifying VRF tables.

        Import RTs of VRF table is matched with RTs from *vpn4_path* and if we
        have any common RTs we import the path into VRF.
        """
        assert (vpn_path.route_family in
                (Vpnv4Path.ROUTE_FAMILY, Vpnv6Path.ROUTE_FAMILY))
        LOG.debug('Importing path %s to qualifying VRFs', vpn_path)

        # If this path has no RTs we are done.
        if not path_rts:
            LOG.info('Encountered a path with no RTs: %s', vpn_path)
            return

        # We match path RTs with all VRFs that are interested in them.
        interested_tables = set()

        # Get route family of VRF to when this VPN Path can be imported to
        route_family = RF_IPv4_UC
        if vpn_path.route_family != RF_IPv4_VPN:
            route_family = RF_IPv6_UC
        for rt in path_rts:
            rt_rf_id = rt + ':' + str(route_family)
            vrf_rt_tables = self._tables_for_rt.get(rt_rf_id)
            if vrf_rt_tables:
                interested_tables.update(vrf_rt_tables)

        if interested_tables:
            # We iterate over all VRF tables that are interested in the RT
            # of the given path and import this path into them.
            route_dist = vpn_path.nlri.route_dist
            for vrf_table in interested_tables:
                if not (vpn_path.source is None
                        and route_dist == vrf_table.vrf_conf.route_dist):
                    update_vrf_dest = vrf_table.import_vpn_path(vpn_path)
                    # Queue the destination for further processing.
                    if update_vrf_dest is not None:
                        self._signal_bus.\
                            dest_changed(update_vrf_dest)
        else:
            # If we do not have any VRF with import RT that match with path RT
            LOG.debug('No VRF table found that imports RTs: %s', path_rts)

    def add_to_vrf(self, route_dist, prefix, next_hop, route_family):
        """Adds `prefix` to VRF identified by `route_dist` with given
         `next_hop`.

        Returns assigned VPN label.
        """
        from ryu.services.protocols.bgp.core import BgpCoreError

        assert route_dist and prefix and next_hop
        if route_family not in (VRF_RF_IPV4, VRF_RF_IPV6):
            raise ValueError('Given route_family %s is not supported.' %
                             route_family)

        vrf_table = None
        table_id = (route_dist, route_family)
        if route_family == VRF_RF_IPV4:
            vrf_table = self._tables.get(table_id)
            if vrf_table is None:
                raise BgpCoreError(desc='VRF table for RD: %s does not '
                                        'exist.' % route_dist)
            if not is_valid_ipv4_prefix(prefix) or not is_valid_ipv4(next_hop):
                raise BgpCoreError(desc='Invalid Ipv4 prefix or nexthop.')
            ip, masklen = prefix.split('/')
            prefix = IPAddrPrefix(int(masklen), ip)
        elif route_family == VRF_RF_IPV6:
            vrf_table = self._tables.get(table_id)
            if vrf_table is None:
                raise BgpCoreError(desc='VRF table for RD: %s does not '
                                        'exist.' % route_dist)
            if not is_valid_ipv6_prefix(prefix) or not is_valid_ipv6(next_hop):
                raise BgpCoreError(desc='Invalid Ipv6 prefix or nexthop.')
            ip6, masklen = prefix.split('/')
            prefix = IP6AddrPrefix(int(masklen), ip6)

        return vrf_table.insert_vrf_path(
            prefix, next_hop=next_hop,
            gen_lbl=True
        )

    def add_to_global_table(self, prefix, nexthop=None,
                            is_withdraw=False):
        src_ver_num = 1
        peer = None
        # set mandatory path attributes
        origin = BGPPathAttributeOrigin(BGP_ATTR_ORIGIN_IGP)
        aspath = BGPPathAttributeAsPath([[]])

        pathattrs = OrderedDict()
        pathattrs[BGP_ATTR_TYPE_ORIGIN] = origin
        pathattrs[BGP_ATTR_TYPE_AS_PATH] = aspath

        net = netaddr.IPNetwork(prefix)
        ip = str(net.ip)
        masklen = net.prefixlen
        if netaddr.valid_ipv4(ip):
            _nlri = IPAddrPrefix(masklen, ip)
            if nexthop is None:
                nexthop = '0.0.0.0'
            p = Ipv4Path
        else:
            _nlri = IP6AddrPrefix(masklen, ip)
            if nexthop is None:
                nexthop = '::'
            p = Ipv6Path

        new_path = p(peer, _nlri, src_ver_num,
                     pattrs=pathattrs, nexthop=nexthop,
                     is_withdraw=is_withdraw)

        # add to global ipv4 table and propagates to neighbors
        self.learn_path(new_path)

    def remove_from_vrf(self, route_dist, prefix, route_family):
        """Removes `prefix` from VRF identified by `route_dist`.

        Returns assigned VPN label.
        """
        from ryu.services.protocols.bgp.core import BgpCoreError
        # Validate given
        if route_family not in (VRF_RF_IPV4, VRF_RF_IPV6):
            raise BgpCoreError(desc='Unsupported route family %s' %
                                    route_family)
        val_ipv4 = route_family == VRF_RF_IPV4\
            and is_valid_ipv4_prefix(prefix)
        val_ipv6 = route_family == VRF_RF_IPV6\
            and is_valid_ipv6_prefix(prefix)

        if not val_ipv4 and not val_ipv6:
            raise BgpCoreError(desc='Invalid prefix or nexthop.')

        table_id = (route_dist, route_family)
        if route_family == VRF_RF_IPV4:
            vrf_table = self._tables.get(table_id)
            if not vrf_table:
                raise BgpCoreError(desc='Vrf for route distinguisher %s does '
                                        'not exist.' % route_dist)
            ip, masklen = prefix.split('/')
            prefix = IPAddrPrefix(int(masklen), ip)
        else:
            vrf_table = self._tables.get(table_id)
            if not vrf_table:
                raise BgpCoreError(desc='Vrf for route distinguisher %s does '
                                        'not exist.' % route_dist)
            ip6, masklen = prefix.split('/')
            prefix = IP6AddrPrefix(int(masklen), ip6)
            # We do not check if we have a path to given prefix, we issue
        # withdrawal. Hence multiple withdrawals have not side effect.
        return vrf_table.insert_vrf_path(prefix, is_withdraw=True)

    def clean_stale_routes(self, peer, route_family=None):
        """Removes old routes from `peer` from `route_family` table.

        Routes/paths version number is compared with `peer`s current version
        number.
        """

        if route_family is not None:
            if route_family not in SUPPORTED_GLOBAL_RF:
                raise ValueError('Given route family %s is not supported.' %
                                 route_family)

            tables = [self._global_tables.get(route_family)]
        else:
            tables = self._global_tables.values()
        for table in tables:
            table.cleanup_paths_for_peer(peer)



# Copyright (c) 2014 Rackspace Hosting
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from ryu.lib import dpid as dpidlib
from ryu.services.protocols.ovsdb import event as ovsdb_event


def match_row(manager, system_id, table, fn):
    def _match_row(tables):
        return next((r for r in tables[table].rows.values()
                     if fn(r)), None)

    request_to_get_tables = ovsdb_event.EventReadRequest(system_id,
                                                         _match_row)
    reply_to_get_tables = manager.send_request(request_to_get_tables)
    return reply_to_get_tables.result


def match_rows(manager, system_id, table, fn):
    def _match_rows(tables):
        return (r for r in tables[table].rows.values() if fn(r))

    request = ovsdb_event.EventReadRequest(system_id, _match_rows)
    reply = manager.send_request(request)
    return reply.result


def row_by_name(manager, system_id, name, table='Bridge', fn=None):
    matched_row = match_row(manager, system_id, table,
                            lambda row: row.name == name)

    if fn is not None:
        return fn(matched_row)

    return matched_row


def get_column_value(manager, table, record, column):
    """
    Example : To get datapath_id from Bridge table
    get_column_value('Bridge', <bridge name>, 'datapath_id').strip('"')
    """
    row = row_by_name(manager, record, table)
    value = getattr(row, column, "")

    if isinstance(value, list) and len(value) == 1:
        value = value[0]

    return str(value)


def get_iface_by_name(manager, system_id, name, fn=None):
    iface = row_by_name(manager, system_id, name, 'Interface')

    if fn is not None:
        return fn(iface)

    return iface


def get_bridge_for_iface_name(manager, system_id, iface_name, fn=None):
    iface = row_by_name(manager, system_id, iface_name, 'Interface')
    port = match_row(manager, system_id, 'Port',
                     lambda x: iface in x.interfaces)
    bridge = match_row(manager, system_id, 'Bridge',
                       lambda x: port in x.ports)

    if fn is not None:
        return fn(bridge)

    return bridge


def get_table(manager, system_id, name):
    def _get_table(tables):
        return tables[name]

    request_to_get_tables = ovsdb_event.EventReadRequest(system_id,
                                                         _get_table)
    reply_to_get_tables = manager.send_request(request_to_get_tables)
    return reply_to_get_tables.result


def get_bridge_by_datapath_id(manager, system_id, datapath_id, fn=None):
    def _match_fn(row):
        row_dpid = dpidlib.str_to_dpid(str(row.datapath_id[0]))
        return row_dpid == datapath_id

    bridge = match_row(manager, system_id, 'Bridge', _match_fn)

    if fn is not None:
        return fn(bridge)

    return bridge


def get_datapath_ids_for_systemd_id(manager, system_id):
    def _get_dp_ids(tables):
        dp_ids = []

        bridges = tables.get('Bridge')

        if not bridges:
            return dp_ids

        for bridge in bridges.rows.values():
            datapath_ids = bridge.datapath_id
            dp_ids.extend(dpidlib.str_to_dpid(dp_id) for dp_id in datapath_ids)

        return dp_ids

    request = ovsdb_event.EventReadRequest(system_id, _get_dp_ids)
    reply = manager.send_request(request)
    return reply.result


def get_bridges_by_system_id(manager, system_id):
    return get_table(manager, system_id, 'Bridge').rows.values()


def bridge_exists(manager, system_id, bridge_name):
    return bool(row_by_name(manager, system_id, bridge_name))


def port_exists(manager, system_id, port_name):
    return bool(row_by_name(manager, system_id, port_name, 'Port'))



# Copyright (C) 2014 Nippon Telegraph and Telephone Corporation.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import print_function

import unittest
import logging
import netaddr
import functools
import inspect

from nose.tools import *

from ryu.lib import addrconv
from ryu.lib import ofctl_v1_0
from ryu.ofproto import ofproto_v1_0, ofproto_v1_0_parser
from ryu.lib import ofctl_v1_2
from ryu.ofproto import ofproto_v1_2, ofproto_v1_2_parser
from ryu.lib import ofctl_v1_3
from ryu.ofproto import ofproto_v1_3, ofproto_v1_3_parser
from ryu.ofproto import ofproto_protocol
from ryu.ofproto import inet
from ryu.tests import test_lib

LOG = logging.getLogger('test_ofctl_v1_2, v1_3')

""" Common Functions """


def _str_to_int(v):
    try:
        return int(v, 0)
    except (ValueError, TypeError):
        return v


def _to_match_eth(value):
    if '/' in value:
        value = value.split('/')
        return value[0], value[1]
    else:
        return value, None


def _to_match_ip(value):
    if '/' in value:
        ip = netaddr.ip.IPNetwork(value)
        ip_addr = str(ip.network)
        ip_mask = str(ip.netmask)
        return ip_addr, ip_mask
    else:
        return value, None


def _to_match_masked_int(value):
    if isinstance(value, str) and '/' in value:
        value = value.split('/')
        return _str_to_int(value[0]), _str_to_int(value[1])
    else:
        return _str_to_int(value), None


def _to_masked_int_str(value):
    v, m = _to_match_masked_int(value)
    v &= m
    return '%d/%d' % (v, m)


conv_of10_to_of12_dict = {
    'dl_dst': 'eth_dst',
    'dl_src': 'eth_src',
    'dl_type': 'eth_type',
    'dl_vlan': 'vlan_vid',
    'nw_src': 'ipv4_src',
    'nw_dst': 'ipv4_dst',
    'nw_proto': 'ip_proto'
}


conv_of12_to_of10_dict = {
    'eth_src': 'dl_src',
    'eth_dst': 'dl_dst',
    'eth_type': 'dl_type',
    'ipv4_dst': 'nw_dst',
    'ipv4_src': 'nw_src',
    'ip_proto': 'nw_proto',
    'vlan_vid': 'dl_vlan',
    'tcp_src': 'tp_src',
    'tcp_dst': 'tp_dst',
    'udp_src': 'tp_src',
    'udp_dst': 'tp_dst'
}

""" Test_ofctl """


class Test_ofctl(unittest.TestCase):

    def __init__(self, methodName):
        super(Test_ofctl, self).__init__(methodName)

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def _test_actions(self, act, test):
        dp = ofproto_protocol.ProtocolDesc(version=test.ver)
        act_type = act["type"]

        # str -> action
        insts = test.to_actions(dp, [act])

        if test.ver == ofproto_v1_0.OFP_VERSION:
            action = insts[0]
            self._equal_str_to_act(action, act, act_type, test)
        else:
            inst = insts[0]
            self._equal_str_to_inst(inst, act, act_type, test)

        # action -> str
        inst_str = test.actions_to_str(insts)
        if test.ver == ofproto_v1_0.OFP_VERSION:
            act_str = inst_str
            self._equal_act_to_str(act_str, act, act_type, test)
        else:
            self._equal_inst_to_str(inst_str, act, act_type, test)

    def _test_match(self, attrs, test):
        dp = ofproto_protocol.ProtocolDesc(version=test.ver)

        # str -> match
        match = test.to_match(dp, attrs)

        for key, value in attrs.items():
            key = self._conv_key(test, key, attrs)
            self._equal_str_to_match(key, value, match, test)

        # match -> str
        match_str = test.match_to_str(match)

        for key, value in attrs.items():
            if key in conv_of12_to_of10_dict:
                key_old = conv_of12_to_of10_dict[key]
            else:
                key_old = key
            self._equal_match_to_str(key_old, value, match_str, test)

    def _equal_str_to_inst(self, inst, act, act_type, test):
        if act_type in test.supported_action:
            cls = test.supported_action[act_type]
        else:
            cls = None
        if act_type == 'GOTO_TABLE':
            ok_(isinstance(inst, cls))
            eq_(inst.table_id, act["table_id"])
        elif act_type == 'WRITE_METADATA':
            ok_(isinstance(inst, cls))
            eq_(inst.metadata, act["metadata"])
            eq_(inst.metadata_mask, act["metadata_mask"])
        elif act_type == 'METER':
            ok_(isinstance(inst, cls))
            eq_(inst.meter_id, act["meter_id"])
        elif act_type == 'WRITE_ACTIONS':
            ok_(isinstance(inst, cls))
            eq_(inst.type, test._ofproto.OFPIT_WRITE_ACTIONS)
            self._equal_str_to_act(inst.actions[0],
                                   act["actions"][0],
                                   act["actions"][0]["type"],
                                   test)
        elif act_type == 'CLEAR_ACTIONS':
            ok_(isinstance(inst, cls))
            eq_(inst.type, test._ofproto.OFPIT_CLEAR_ACTIONS)
        else:
            # APPLY_ACTIONS or Uknown Action Type
            ok_(isinstance(inst, test._parser.OFPInstructionActions))
            eq_(inst.type, test._ofproto.OFPIT_APPLY_ACTIONS)
            self._equal_str_to_act(inst.actions[0], act,
                                   act_type, test)

    def _equal_str_to_act(self, action, act, act_type, test):
        if act_type in test.supported_action:
            cls = test.supported_action[act_type]
        else:
            cls = None
        ok_(isinstance(action, cls))
        if act_type == 'OUTPUT':
            eq_(action.port, act["port"])
        elif act_type == 'SET_VLAN_VID':
            eq_(action.vlan_vid, act["vlan_vid"])
        elif act_type == 'SET_VLAN_PCP':
            eq_(action.vlan_pcp, act["vlan_pcp"])
        elif act_type == 'SET_DL_SRC':
            eq_(addrconv.mac.bin_to_text(action.dl_addr),
                act["dl_src"])
        elif act_type == 'SET_DL_DST':
            eq_(addrconv.mac.bin_to_text(action.dl_addr),
                act["dl_dst"])
        elif act_type == 'SET_NW_SRC':
            ip = netaddr.ip.IPAddress(action.nw_addr)
            eq_(str(ip), act["nw_src"])
        elif act_type == 'SET_NW_DST':
            ip = netaddr.ip.IPAddress(action.nw_addr)
            eq_(str(ip), act["nw_dst"])
        elif act_type == 'SET_NW_TOS':
            eq_(action.tos, act["nw_tos"])
        elif act_type == 'SET_TP_SRC':
            eq_(action.tp, act["tp_src"])
        elif act_type == 'SET_TP_DST':
            eq_(action.tp, act["tp_dst"])
        elif act_type == 'ENQUEUE':
            eq_(action.queue_id, act["queue_id"])
            eq_(action.port, act["port"])
        elif act_type == 'SET_MPLS_TTL':
            eq_(action.mpls_ttl, act["mpls_ttl"])
        elif act_type in ['PUSH_VLAN', 'PUSH_MPLS',
                          'POP_MPLS', 'PUSH_PBB']:
            eq_(action.ethertype, act["ethertype"])
        elif act_type == 'SET_QUEUE':
            eq_(action.queue_id, act["queue_id"])
        elif act_type == 'GROUP':
            eq_(action.group_id, act["group_id"])
        elif act_type == 'SET_NW_TTL':
            eq_(action.nw_ttl, act["nw_ttl"])
        elif act_type == 'SET_FIELD':
            eq_(action.key, act['field'])
            eq_(action.value, act['value'])
        elif act_type in ['STRIP_VLAN', 'COPY_TTL_OUT',
                          'COPY_TTL_IN', 'DEC_MPLS_TTL',
                          'POP_VLAN', 'DEC_NW_TTL', 'POP_PBB']:
            pass
        else:  # Uknown Action Type
            assert False

    def _equal_inst_to_str(self, inst_str, act, act_type, test):
        if act_type == 'WRITE_ACTIONS':
            act_str = inst_str[0]["WRITE_ACTIONS"]
            act = act["actions"][0]
            act_type = act["type"]
            self._equal_act_to_str(act_str, act, act_type, test)
        else:
            inst_str_list = inst_str[0].split(':', 1)
            eq_(inst_str_list[0], act_type)
            if act_type == 'GOTO_TABLE':
                eq_(int(inst_str_list[1]), act["table_id"])
            elif act_type == 'WRITE_METADATA':
                met = inst_str_list[1].split('/')
                eq_(int(met[0], 16), act["metadata"])
                eq_(int(met[1], 16), act["metadata_mask"])
            elif act_type == 'METER':
                eq_(int(inst_str_list[1]), act["meter_id"])
            elif act_type == 'CLEAR_ACTIONS':
                pass
            else:
                # APPLY_ACTIONS
                act_str = inst_str
                self._equal_act_to_str(act_str, act, act_type, test)

    def _equal_act_to_str(self, act_str, act, act_type, test):
        act_str_list = act_str[0].split(':', 1)
        eq_(act_str_list[0], act_type)
        if act_type == 'OUTPUT':
            eq_(int(act_str_list[1]), act["port"])
        elif act_type == 'SET_VLAN_VID':
            eq_(int(act_str_list[1]), act["vlan_vid"])
        elif act_type == 'SET_VLAN_PCP':
            eq_(int(act_str_list[1]), act["vlan_pcp"])
        elif act_type == 'SET_DL_SRC':
            eq_(act_str_list[1], act["dl_src"])
        elif act_type == 'SET_DL_DST':
            eq_(act_str_list[1], act["dl_dst"])
        elif act_type == 'SET_NW_SRC':
            eq_(act_str_list[1], act["nw_src"])
        elif act_type == 'SET_NW_DST':
            eq_(act_str_list[1], act["nw_dst"])
        elif act_type == 'SET_NW_TOS':
            eq_(int(act_str_list[1]), act["nw_tos"])
        elif act_type == 'SET_TP_SRC':
            eq_(int(act_str_list[1]), act["tp_src"])
        elif act_type == 'SET_TP_DST':
            eq_(int(act_str_list[1]), act["tp_dst"])
        elif act_type == 'ENQUEUE':
            enq = act_str_list[1].split(':')
            eq_(int(enq[0], 10), act["port"])
            eq_(int(enq[1], 10), act["queue_id"])
        elif act_type == 'SET_MPLS_TTL':
            eq_(int(act_str_list[1]), act["mpls_ttl"])
        elif act_type == 'PUSH_VLAN':
            eq_(int(act_str_list[1]), act["ethertype"])
        elif act_type == 'PUSH_MPLS':
            eq_(int(act_str_list[1]), act["ethertype"])
        elif act_type == 'POP_MPLS':
            eq_(int(act_str_list[1]), act["ethertype"])
        elif act_type == 'SET_QUEUE':
            eq_(int(act_str_list[1]), act["queue_id"])
        elif act_type == 'GROUP':
            eq_(int(act_str_list[1]), act["group_id"])
        elif act_type == 'SET_NW_TTL':
            eq_(int(act_str_list[1]), act["nw_ttl"])
        elif act_type == 'SET_FIELD':
            field, value = act_str_list[1].split(':')
            eq_(field.strip(' {'), act["field"])
            eq_(int(value.strip('} ')), act["value"])
        elif act_type == 'PUSH_PBB':
            eq_(int(act_str_list[1]), act["ethertype"])
        elif act_type in ['STRIP_VLAN', 'COPY_TTL_OUT',
                          'COPY_TTL_IN', 'DEC_MPLS_TTL',
                          'POP_VLAN', 'DEC_NW_TTL', 'POP_PBB']:
            pass
        else:
            assert False

    def _equal_str_to_match(self, key, value, match, test):
        field_value = self._get_field_value(test, key, match)

        if key in ['eth_src', 'eth_dst', 'arp_sha', 'arp_tha']:
            # MAC address
            eth, mask = _to_match_eth(value)
            if mask is not None:
                # with mask
                for i in range(0, len(mask)):
                    if mask[i] == 'f':
                        eq_(eth[i], field_value[0][i])
                eq_(mask, field_value[1])
            else:
                # without mask
                eq_(eth, field_value)
            return
        elif key in ['dl_src', 'dl_dst']:
            eth, mask = _to_match_eth(value)
            field_value = addrconv.mac.bin_to_text(field_value)
            eq_(eth, field_value)
            return
        elif key in ['ipv4_src', 'ipv4_dst', 'arp_spa', 'arp_tpa']:
            # IPv4 address
            ipv4, mask = _to_match_ip(value)
            if mask is not None:
                # with mask
                eq_(ipv4, field_value[0])
                eq_(mask, field_value[1])
            else:
                # without mask
                eq_(ipv4, field_value)
            return
        elif key in ['nw_src', 'nw_dst']:
            # IPv4 address
            ipv4, mask = _to_match_ip(value)
            field_value = _to_match_ip(field_value)
            if mask is not None:
                # with mask
                eq_(ipv4, field_value[0])
                eq_(mask, field_value[1])
            else:
                # without mask
                eq_(ipv4, field_value[0])
            return
        elif key in ['ipv6_src', 'ipv6_dst']:
            # IPv6 address
            ipv6, mask = _to_match_ip(value)
            if mask is not None:
                # with mask
                eq_(ipv6, field_value[0])
                eq_(mask, field_value[1])
            else:
                # without mask
                eq_(ipv6, field_value)
            return
        elif key == 'vlan_vid':
            if test.ver == ofproto_v1_0.OFP_VERSION:
                eq_(value, field_value)
            else:
                eq_(test.expected_value['vlan_vid'][
                    value]['to_match'], field_value)
            return
        else:
            if isinstance(value, str) and '/' in value:
                # with mask
                value, mask = _to_match_masked_int(value)
                value &= mask
                eq_(value, field_value[0])
                eq_(mask, field_value[1])
            else:
                # without mask
                eq_(_str_to_int(value), field_value)
            return

    def _equal_match_to_str(self, key, value, match_str, test):
        field_value = match_str[key]
        if key in ['dl_src', 'dl_dst', 'arp_sha', 'arp_tha']:
            # MAC address
            eth, mask = _to_match_eth(value)
            if mask is not None:
                # with mask
                field_value = field_value.split('/')
                for i in range(0, len(mask)):
                    if mask[i] == 'f':
                        eq_(eth[i], field_value[0][i])
                eq_(mask, field_value[1])
            else:
                # without mask
                eq_(eth, field_value)
            return
        elif key in['nw_src', 'nw_dst', 'arp_spa', 'arp_tpa']:
            # IPv4 address
            if test.ver == ofproto_v1_0.OFP_VERSION:
                ipv4, mask = _to_match_ip(value)
                field_value = _to_match_ip(field_value)
                if mask is not None:
                    # with mask
                    eq_(ipv4, field_value[0])
                    eq_(mask, field_value[1])
                else:
                    # without mask
                    eq_(ipv4, field_value[0])
            else:
                ipv4, mask = _to_match_ip(value)
                if mask is not None:
                    # with mask
                    field_value = field_value.split('/')
                    eq_(ipv4, field_value[0])
                    eq_(mask, field_value[1])
                else:
                    # without mask
                    eq_(ipv4, field_value)
            return
        elif key in ['ipv6_src', 'ipv6_dst']:
            # IPv6 address
            ipv6, mask = _to_match_ip(value)
            if mask is not None:
                # with mask
                field_value = field_value.split('/')
                eq_(ipv6, field_value[0])
                eq_(mask, field_value[1])
            else:
                # without mask
                eq_(ipv6, field_value)
            return
        elif key == 'dl_vlan':
            if test.ver == ofproto_v1_0.OFP_VERSION:
                eq_(value, field_value)
            else:
                eq_(test.expected_value['vlan_vid'][
                    value]['to_str'], field_value)
            return
        else:
            if isinstance(value, str) and '/' in value:
                # with mask
                value = _to_masked_int_str(value)
                eq_(value, field_value)
            else:
                # without mask
                eq_(_str_to_int(value), field_value)
            return

    def _conv_key(self, test, key, attrs):
        if test.ver != ofproto_v1_0.OFP_VERSION:
            if key in conv_of10_to_of12_dict:
                # For old field name
                key = conv_of10_to_of12_dict[key]
            elif key == 'tp_src' or key == 'tp_dst':
                # TCP/UDP port
                conv = {inet.IPPROTO_TCP: {'tp_src': 'tcp_src',
                                           'tp_dst': 'tcp_dst'},
                        inet.IPPROTO_UDP: {'tp_src': 'udp_src',
                                           'tp_dst': 'udp_dst'}}
                ip_proto = attrs.get('nw_proto', attrs.get('ip_proto', 0))
                key = conv[ip_proto][key]

        return key

    def _get_field_value(self, test, key, match):
        if test.ver == ofproto_v1_0.OFP_VERSION:
            members = inspect.getmembers(match)
            for member in members:
                if member[0] == key:
                    field_value = member[1]
                elif member[0] == 'wildcards':
                    wildcards = member[1]
            if key == 'nw_src':
                field_value = test.nw_src_to_str(wildcards, field_value)
            elif key == 'nw_dst':
                field_value = test.nw_dst_to_str(wildcards, field_value)
        else:
            field_value = match[key]

        return field_value


class test_data_base(object):
    # followings must be an attribute of subclass.
    # _ofctl
    # _ofproto

    def __init__(self):
        self.ver = self._ofproto.OFP_VERSION
        self.to_match = self._ofctl.to_match
        self.match_to_str = self._ofctl.match_to_str
        self.to_actions = self._ofctl.to_actions
        self.actions_to_str = self._ofctl.actions_to_str


class test_data_v1_0(test_data_base):
    """ Test_data for of_v1_0 """
    _ofctl = ofctl_v1_0
    _ofproto = ofproto_v1_0
    _parser = ofproto_v1_0_parser

    def __init__(self):
        super(test_data_v1_0, self).__init__()
        self.nw_src_to_str = self._ofctl.nw_src_to_str
        self.nw_dst_to_str = self._ofctl.nw_dst_to_str
        self.supported_action = {}
        self.act_list = [
            {'type': 'OUTPUT', 'port': 3},
            {'type': 'SET_VLAN_VID', 'vlan_vid': 5},
            {'type': 'SET_VLAN_PCP', 'vlan_pcp': 3},
            {'type': 'STRIP_VLAN'},
            {'type': 'SET_DL_SRC', 'dl_src': 'aa:bb:cc:11:22:33'},
            {'type': 'SET_DL_DST', 'dl_dst': 'aa:bb:cc:11:22:33'},
            {'type': 'SET_NW_SRC', 'nw_src': '10.0.0.1'},
            {'type': 'SET_NW_DST', 'nw_dst': '10.0.0.1'},
            {'type': 'SET_NW_TOS', 'nw_tos': 184},
            {'type': 'SET_TP_SRC', 'tp_src': 8080},
            {'type': 'SET_TP_DST', 'tp_dst': 8080},
            {'type': 'ENQUEUE', 'queue_id': 3, 'port': 1}
        ]
        self.attr_list = [
            {'in_port': 7},
            {'dl_src': 'aa:bb:cc:11:22:33'},
            {'dl_dst': 'aa:bb:cc:11:22:33'},
            {'dl_vlan': 5},
            {'dl_vlan_pcp': 3},
            {'dl_type': 123},
            {'nw_tos': 16},
            {'nw_proto': 5},
            {'nw_src': '192.168.0.1'},
            {'nw_src': '192.168.0.1/24'},
            {'nw_dst': '192.168.0.1'},
            {'nw_dst': '192.168.0.1/24'},
            {'tp_src': 1},
            {'tp_dst': 2}
        ]
        self.set_action()

    def set_action(self):
        self.supported_action.update(
            {
                'OUTPUT': self._parser.OFPActionOutput,
                'SET_VLAN_VID': self._parser.OFPActionVlanVid,
                'SET_VLAN_PCP': self._parser.OFPActionVlanPcp,
                'STRIP_VLAN': self._parser.OFPActionStripVlan,
                'SET_DL_SRC': self._parser.OFPActionSetDlSrc,
                'SET_DL_DST': self._parser.OFPActionSetDlDst,
                'SET_NW_SRC': self._parser.OFPActionSetNwSrc,
                'SET_NW_DST': self._parser.OFPActionSetNwDst,
                'SET_NW_TOS': self._parser.OFPActionSetNwTos,
                'SET_TP_SRC': self._parser.OFPActionSetTpSrc,
                'SET_TP_DST': self._parser.OFPActionSetTpDst,
                'ENQUEUE': self._parser.OFPActionEnqueue
            })


class test_data_v1_2(test_data_base):
    """ Test_data for of_v1_2 """
    _ofctl = ofctl_v1_2
    _ofproto = ofproto_v1_2
    _parser = ofproto_v1_2_parser

    def __init__(self):
        super(test_data_v1_2, self).__init__()
        self.supported_action = {}
        self.act_list = [
            {'type': 'OUTPUT', 'port': 3},
            {'type': 'COPY_TTL_OUT'},
            {'type': 'COPY_TTL_IN'},
            {'type': 'SET_MPLS_TTL', 'mpls_ttl': 64},
            {'type': 'DEC_MPLS_TTL'},
            {'type': 'PUSH_VLAN', 'ethertype': 0x0800},
            {'type': 'POP_VLAN'},
            {'type': 'PUSH_MPLS', 'ethertype': 0x0800},
            {'type': 'POP_MPLS', 'ethertype': 0x0800},
            {'type': 'SET_QUEUE', 'queue_id': 7},
            {'type': 'GROUP', 'group_id': 5},
            {'type': 'SET_NW_TTL', 'nw_ttl': 64},
            {'type': 'DEC_NW_TTL'},
            {"type": "CLEAR_ACTIONS"},
            {"type": "WRITE_ACTIONS",
             "actions": [{"type": "OUTPUT", "port": 4}]},
            {'type': 'GOTO_TABLE', 'table_id': 8},
            {'type': 'WRITE_METADATA', 'metadata': 8,
             'metadata_mask': (1 << 64) - 1},
        ]
        self.attr_list = [
            {'in_port': 7},
            {'in_phy_port': 5, 'in_port': 3},
            {'metadata': 12345},
            {'metadata': '0x1212121212121212'},
            {'metadata': '0x19af28be37fa91b/0x1010101010101010'},
            {'dl_src': "aa:bb:cc:11:22:33"},
            {'dl_src': "aa:bb:cc:11:22:33/00:00:00:00:ff:ff"},
            {'dl_dst': "aa:bb:cc:11:22:33"},
            {'dl_dst': "aa:bb:cc:11:22:33/00:00:00:00:ff:ff"},
            {'dl_type': 123},
            {'eth_src': "aa:bb:cc:11:22:33"},
            {'eth_src': "aa:bb:cc:11:22:33/00:00:00:00:ff:ff"},
            {'eth_dst': "aa:bb:cc:11:22:33"},
            {'eth_dst': "aa:bb:cc:11:22:33/00:00:00:00:ff:ff"},
            {'eth_type': 0x800},
            {'dl_vlan': 0},
            {'dl_vlan': 3},
            {'dl_vlan': 4095},
            {'dl_vlan': "0"},
            {'dl_vlan': "3"},
            {'dl_vlan': "4095"},
            {'dl_vlan': "0x0000"},
            {'dl_vlan': "0x0003"},
            {'dl_vlan': "0x0fff"},
            {'dl_vlan': "0x1000"},
            {'dl_vlan': "0x1003"},
            {'dl_vlan': "0x1fff"},
            {'dl_vlan': "4096/4096"},
            {'dl_vlan': "4096/4097"},
            {'dl_vlan': "2744/2748"},
            {'dl_vlan': "2748/2748"},
            {'dl_vlan': "2748/2749"},
            {'dl_vlan': "0x1000/0x1000"},
            {'dl_vlan': "0x1000/0x1001"},
            {'dl_vlan': "0x0ab8/0x0abc"},
            {'dl_vlan': "0x0abc/0x0abc"},
            {'dl_vlan': "0x0abc/0x0abd"},
            {'vlan_pcp': 3, 'vlan_vid': 3},
            {'ip_dscp': 3, 'eth_type': 0x0800},
            {'ip_ecn': 4, 'eth_type': 0x86dd},
            {'nw_src': "192.168.0.1", 'eth_type': 0x0800},
            {'nw_src': "192.168.0.1/24", 'eth_type': 0x0800},
            {'nw_src': "192.168.10.10/255.255.0.0", 'eth_type': 0x0800},
            {'nw_dst': "192.168.0.1", 'eth_type': 0x0800},
            {'nw_dst': "192.168.0.1/24", 'eth_type': 0x0800},
            {'nw_dst': "192.168.10.10/255.255.255.0"},
            {'nw_proto': 5, 'eth_type': 0x0800},
            {'ip_proto': 5, 'eth_type': 0x86dd},
            {'ipv4_src': "192.168.0.1", 'eth_type': 0x0800},
            {'ipv4_src': "192.168.0.1/24", 'eth_type': 0x0800},
            {'ipv4_src': "192.168.10.10/255.255.0.0", 'eth_type': 0x0800},
            {'ipv4_dst': "192.168.0.1", 'eth_type': 0x0800},
            {'ipv4_dst': "192.168.0.1/24", 'eth_type': 0x0800},
            {'ipv4_dst': "192.168.10.10/255.255.255.0", 'eth_type': 0x0800},
            {'tp_src': 1, 'ip_proto': 6},
            {'tp_dst': 2, 'ip_proto': 6},
            {'tp_src': 3, 'ip_proto': 17},
            {'tp_dst': 4, 'ip_proto': 17},
            {'vlan_vid': 0},
            {'vlan_vid': 3},
            {'vlan_vid': 4095},
            {'vlan_vid': "0"},
            {'vlan_vid': "3"},
            {'vlan_vid': "4095"},
            {'vlan_vid': "0x0000"},
            {'vlan_vid': "0x0003"},
            {'vlan_vid': "0x0fff"},
            {'vlan_vid': "0x1000"},
            {'vlan_vid': "0x1003"},
            {'vlan_vid': "0x1fff"},
            {'vlan_vid': "4096/4096"},
            {'vlan_vid': "4096/4097"},
            {'vlan_vid': "2744/2748"},
            {'vlan_vid': "2748/2748"},
            {'vlan_vid': "2748/2749"},
            {'vlan_vid': "0x1000/0x1000"},
            {'vlan_vid': "0x1000/0x1001"},
            {'vlan_vid': "0x0ab8/0x0abc"},
            {'vlan_vid': "0x0abc/0x0abc"},
            {'vlan_vid': "0x0abc/0x0abd"},
            {'tcp_src': 3, 'ip_proto': 6},
            {'tcp_dst': 5, 'ip_proto': 6},
            {'udp_src': 2, 'ip_proto': 17},
            {'udp_dst': 6, 'ip_proto': 17},
            {'sctp_src': 99, 'ip_proto': 132},
            {'sctp_dst': 99, 'ip_proto': 132},
            {'icmpv4_type': 5, 'ip_proto': 1},
            {'icmpv4_code': 6, 'ip_proto': 1},
            {'arp_op': 3, 'eth_type': 0x0806},
            {'arp_spa': "192.168.0.11", 'eth_type': 0x0806},
            {'arp_spa': "192.168.0.22/24", 'eth_type': 0x0806},
            {'arp_tpa': "192.168.0.33", 'eth_type': 0x0806},
            {'arp_tpa': "192.168.0.44/24", 'eth_type': 0x0806},
            {'arp_sha': "aa:bb:cc:11:22:33", 'eth_type': 0x0806},
            {'arp_sha': "aa:bb:cc:11:22:33/00:00:00:00:ff:ff",
                'eth_type': 0x0806},
            {'arp_tha': "aa:bb:cc:11:22:33", 'eth_type': 0x0806},
            {'arp_tha': "aa:bb:cc:11:22:33/00:00:00:00:ff:ff",
                'eth_type': 0x0806},
            {'ipv6_src': '2001::aaaa:bbbb:cccc:1111', 'eth_type': 0x86dd},
            {'ipv6_src': '2001::aaaa:bbbb:cccc:1111/64', 'eth_type': 0x86dd},
            {'ipv6_dst': '2001::ffff:cccc:bbbb:1111', 'eth_type': 0x86dd},
            {'ipv6_dst': '2001::ffff:cccc:bbbb:1111/64', 'eth_type': 0x86dd},
            {'ipv6_flabel': 2, 'eth_type': 0x86dd},
            {'icmpv6_type': 3, 'ip_proto': 58},
            {'icmpv6_code': 4, 'ip_proto': 58},
            {'ipv6_nd_target': '2001::ffff:cccc:bbbb:1111',
                'icmpv6_type': 135, 'ip_proto': 58},
            {'ipv6_nd_sll': "aa:bb:cc:11:22:33",
                'icmpv6_type': 135, 'ip_proto': 58},
            {'ipv6_nd_tll': "aa:bb:cc:11:22:33",
                'icmpv6_type': 136, 'ip_proto': 58},
            {'mpls_label': 3, 'eth_type': 0x8848},
            {'mpls_tc': 2, 'eth_type': 0x8848}
        ]
        self.supported_action.update(
            {
                'OUTPUT': self._parser.OFPActionOutput,
                'COPY_TTL_OUT': self._parser.OFPActionCopyTtlOut,
                'COPY_TTL_IN': self._parser.OFPActionCopyTtlIn,
                'SET_MPLS_TTL': self._parser.OFPActionSetMplsTtl,
                'DEC_MPLS_TTL': self._parser.OFPActionDecMplsTtl,
                'PUSH_VLAN': self._parser.OFPActionPushVlan,
                'POP_VLAN': self._parser.OFPActionPopVlan,
                'PUSH_MPLS': self._parser.OFPActionPushMpls,
                'POP_MPLS': self._parser.OFPActionPopMpls,
                'SET_QUEUE': self._parser.OFPActionSetQueue,
                'GROUP': self._parser.OFPActionGroup,
                'SET_NW_TTL': self._parser.OFPActionSetNwTtl,
                'DEC_NW_TTL': self._parser.OFPActionDecNwTtl,
                'SET_FIELD': self._parser.OFPActionSetField,
                'GOTO_TABLE': self._parser.OFPInstructionGotoTable,
                'WRITE_METADATA': self._parser.OFPInstructionWriteMetadata,
                'WRITE_ACTIONS': self._parser.OFPInstructionActions,
                'CLEAR_ACTIONS': self._parser.OFPInstructionActions,
            })
        self.set_expected_value()

    def set_expected_value(self):
        vid_present = self._ofproto.OFPVID_PRESENT
        self.expected_value = {
            "vlan_vid": {
                0: {"to_match": 0 | vid_present, "to_str": "0"},
                3: {"to_match": 3 | vid_present, "to_str": "3"},
                4095: {"to_match": 4095 | vid_present, "to_str": "4095"},
                "0": {"to_match": 0 | vid_present, "to_str": "0"},
                "3": {"to_match": 3 | vid_present, "to_str": "3"},
                "4095": {"to_match": 4095 | vid_present, "to_str": "4095"},
                "0x0000": {"to_match": 0x0000, "to_str": "0x0000"},
                "0x0003": {"to_match": 0x0003, "to_str": "0x0003"},
                "0x0fff": {"to_match": 0x0fff, "to_str": "0x0fff"},
                "0x1000": {"to_match": 0x1000, "to_str": "0"},
                "0x1003": {"to_match": 0x1003, "to_str": "3"},
                "0x1fff": {"to_match": 0x1fff, "to_str": "4095"},
                "4096/4096": {"to_match": (4096, 4096),
                              "to_str": "0x1000/0x1000"},
                "4096/4097": {"to_match": (4096, 4097),
                              "to_str": "0x1000/0x1001"},
                "2744/2748": {"to_match": (2744, 2748),
                              "to_str": "0x0ab8/0x0abc"},
                "2748/2748": {"to_match": (2748, 2748),
                              "to_str": "0x0abc/0x0abc"},
                "2748/2749": {"to_match": (2748, 2749),
                              "to_str": "0x0abc/0x0abd"},
                "0x1000/0x1000": {"to_match": (0x1000, 0x1000),
                                  "to_str": "0x1000/0x1000"},
                "0x1000/0x1001": {"to_match": (0x1000, 0x1001),
                                  "to_str": "0x1000/0x1001"},
                "0x0ab8/0x0abc": {"to_match": (0x0ab8, 0x0abc),
                                  "to_str": "0x0ab8/0x0abc"},
                "0x0abc/0x0abc": {"to_match": (0x0abc, 0x0abc),
                                  "to_str": "0x0abc/0x0abc"},
                "0x0abc/0x0abd": {"to_match": (0x0abc, 0x0abd),
                                  "to_str": "0x0abc/0x0abd"}
            }
        }


class test_data_v1_3(test_data_v1_2):
    """ Test_data for of_v1_3 """
    _ofctl = ofctl_v1_3
    _ofproto = ofproto_v1_3
    _parser = ofproto_v1_3_parser

    def __init__(self):
        super(test_data_v1_3, self).__init__()
        self.act_list.extend(
            [
                {'type': 'PUSH_PBB', 'ethertype': 0x0800},
                {'type': 'POP_PBB'},
                {'type': 'METER', 'meter_id': 3},
            ]
        )
        self.attr_list.extend(
            [
                {'mpls_bos': 3, 'eth_type': 0x8848},
                {'pbb_isid': 5, 'eth_type': 0x88E7},
                {'pbb_isid': "0x05", 'eth_type': 0x88E7},
                {'pbb_isid': "0x05/0xff", 'eth_type': 0x88E7},
                {'tunnel_id': 7},
                {'tunnel_id': "0x07"},
                {'tunnel_id': "0x07/0xff"},
                {'ipv6_exthdr': 3, 'eth_type': 0x86dd},
                {'ipv6_exthdr': "0x40", 'eth_type': 0x86dd},
                {'ipv6_exthdr': "0x40/0x1F0", 'eth_type': 0x86dd},
            ]
        )
        self.supported_action.update(
            {
                'PUSH_PBB': self._parser.OFPActionPushPbb,
                'POP_PBB': self._parser.OFPActionPopPbb,
                'METER': self._parser.OFPInstructionMeter,
            })
        self.set_expected_value()


def _add_tests_actions(cls):
    for act in cls.act_list:
        method_name = 'test_' + str(cls.ver) + '_' + act["type"] + '_action'

        def _run(self, name, act, cls):
            print('processing %s ...' % name)
            cls_ = Test_ofctl(name)
            cls_._test_actions(act, cls)
        print('adding %s ...' % method_name)
        func = functools.partial(_run, name=method_name, act=act, cls=cls)
        test_lib.add_method(Test_ofctl, method_name, func)


def _add_tests_match(cls):
    for attr in cls.attr_list:
        for key, value in attr.items():
            method_name = 'test_' + \
                str(cls.ver) + '_' + key + '_' + str(
                    value) + str(type(value)) + '_match'

            def _run(self, name, attr, cls):
                print('processing %s ...' % name)
                cls_ = Test_ofctl(name)
                cls_._test_match(attr, cls)
            print('adding %s ...' % method_name)
            func = functools.partial(
                _run, name=method_name, attr=attr, cls=cls)
            test_lib.add_method(Test_ofctl, method_name, func)


""" Test case """

# for of10
cls = test_data_v1_0()
_add_tests_actions(cls)
_add_tests_match(cls)

# for of12
cls = test_data_v1_2()
_add_tests_actions(cls)
_add_tests_match(cls)

# for of13
cls = test_data_v1_3()
_add_tests_actions(cls)
_add_tests_match(cls)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# FIXME: needs unittests
# FIXME: needs checks for out-of-order, concurrency, etc as attributes
from tinyrpc.exc import RPCError

class RPCServer(object):
    """High level RPC server.

    :param transport: The :py:class:`~tinyrpc.transports.RPCTransport` to use.
    :param protocol: The :py:class:`~tinyrpc.RPCProtocol` to use.
    :param dispatcher: The :py:class:`~tinyrpc.dispatch.RPCDispatcher` to use.
    """
    def __init__(self, transport, protocol, dispatcher):
        self.transport = transport
        self.protocol = protocol
        self.dispatcher = dispatcher

    def serve_forever(self):
        """Handle requests forever.

        Starts the server loop in which the transport will be polled for a new
        message.

        After a new message has arrived,
        :py:func:`~tinyrpc.server.RPCServer._spawn` is called with a handler
        function and arguments to handle the request.

        The handler function will try to decode the message using the supplied
        protocol, if that fails, an error response will be sent. After decoding
        the message, the dispatcher will be asked to handle the resultung
        request and the return value (either an error or a result) will be sent
        back to the client using the transport.

        After calling :py:func:`~tinyrpc.server.RPCServer._spawn`, the server
        will fetch the next message and repeat.
        """
        while True:
            context, message = self.transport.receive_message()

            # assuming protocol is threadsafe and dispatcher is theadsafe, as
            # long as its immutable

            def handle_message(context, message):
                try:
                    request = self.protocol.parse_request(message)
                except RPCError as e:
                    response = e.error_respond()
                else:
                    response = self.dispatcher.dispatch(request)

                # send reply
                self.transport.send_reply(context, response.serialize())

            self._spawn(handle_message, context, message)

    def _spawn(self, func, *args, **kwargs):
        """Spawn a handler function.

        This function is overridden in subclasses to provide concurrency.

        In the base implementation, it simply calls the supplied function
        ``func`` with ``*args`` and ``**kwargs``. This results in a
        single-threaded, single-process, synchronous server.

        :param func: A callable to call.
        :param args: Arguments to ``func``.
        :param kwargs: Keyword arguments to ``func``.
        """
        func(*args, **kwargs)



# Copyright (C) 2013 Nippon Telegraph and Telephone Corporation.
# Copyright (C) 2013 YAMAMOTO Takashi <yamamoto at valinux co jp>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import time
import unittest
from nose.tools import raises

from ryu.lib import hub
hub.patch()


class MyException(BaseException):
    pass


class Test_hub(unittest.TestCase):
    """ Test case for ryu.lib.hub
    """

    def setUp(self):
        pass

    def tearDown(self):
        pass

    # we want to test timeout first because the rest of tests rely on it.
    # thus test_0_ prefix.

    @raises(hub.Timeout)
    def test_0_timeout1(self):
        with hub.Timeout(0.1):
            hub.sleep(1)

    @raises(MyException)
    def test_0_timeout2(self):
        with hub.Timeout(0.1, MyException):
            hub.sleep(1)

    def test_0_timeout3(self):
        with hub.Timeout(1):
            hub.sleep(0.1)
        # sleep some more to ensure timer cancelation
        hub.sleep(2)

    def test_spawn_event1(self):
        def _child(ev, result):
            hub.sleep(1)
            result.append(1)
            ev.set()

        ev = hub.Event()
        result = []
        with hub.Timeout(2):
            hub.spawn(_child, ev, result)
            ev.wait()
        assert len(result) == 1

    def test_spawn_event2(self):
        def _child(ev, result):
            hub.sleep(1)
            result.append(1)
            ev.set()

        ev = hub.Event()
        result = []
        with hub.Timeout(2):
            t = hub.spawn(_child, ev, result)
            ev.wait(timeout=0.5)
        assert len(result) == 0
        ev.wait()
        assert len(result) == 1

    def test_spawn_event3(self):
        def _child(ev, ev2, result):
            ev2.wait()
            hub.sleep(0.5)
            result.append(1)
            ev.set()

        ev = hub.Event()
        ev2 = hub.Event()
        result = []
        with hub.Timeout(2):
            hub.spawn(_child, ev, ev2, result)
            hub.spawn(_child, ev, ev2, result)
            hub.sleep(0.5)
            ev2.set()  # this should wake up the above created two threads
            ev.wait(timeout=1)
        assert len(result) == 2

    def test_spawn_select1(self):
        import select
        import socket

        def _child(s1):
            hub.sleep(0.5)
            s1.send(b"hoge")

        s1, s2 = socket.socketpair()
        with hub.Timeout(1):
            hub.spawn(_child, s1)
            select.select([s2.fileno()], [], [])
            select.select([s2.fileno()], [], [])  # return immediately

    @raises(MyException)
    def test_select1(self):
        import select
        import socket

        s1, s2 = socket.socketpair()
        with hub.Timeout(1, MyException):
            select.select([s2.fileno()], [], [])

    def test_select2(self):
        import select

        with hub.Timeout(1, MyException):
            select.select([], [], [], 0)  # timeout immediately

    def test_select3(self):
        import select
        import socket

        s1, s2 = socket.socketpair()
        with hub.Timeout(1, MyException):
            list = [s1.fileno(), s2.fileno()]
            rlist, wlist, xlist = select.select(list, list, list)
            assert not s1.fileno() in rlist
            assert not s2.fileno() in rlist
            # the following two assertions are commented out because one of
            # them fails with eventlet-patched select.
            #       assert s1.fileno() in wlist
            #       assert s2.fileno() in wlist
            # note: eventlet-patched select returns at most one file.
            assert (s1.fileno() in wlist) or (s2.fileno() in wlist)
            assert not s1.fileno() in xlist
            assert not s2.fileno() in xlist

    def test_spawn_joinall(self):
        def _child(ev2, result):
            ev2.wait()
            hub.sleep(0.5)
            result.append(1)
            raise BaseException("this exception should not be propagated")

        ev2 = hub.Event()
        threads = []
        result = []
        with hub.Timeout(2):
            threads.append(hub.spawn(_child, ev2, result))
            threads.append(hub.spawn(_child, ev2, result))
            hub.sleep(0.5)
            ev2.set()  # this should wake up the above created two threads
            hub.joinall(threads)
        assert len(result) == 2

    def test_spawn_kill_joinall(self):
        def _child(ev2, result):
            ev2.wait()
            result.append(1)

        ev2 = hub.Event()
        threads = []
        result = []
        with hub.Timeout(2):
            threads.append(hub.spawn(_child, ev2, result))
            threads.append(hub.spawn(_child, ev2, result))
            hub.sleep(0.5)
            for t in threads:
                hub.kill(t)
            hub.joinall(threads)
        assert len(result) == 0

    def test_spawn_kill_nowait_joinall(self):
        # XXX this test relies on the scheduling behaviour.
        # the intention here is, killing threads before they get active.

        def _child(result):
            result.append(1)

        threads = []
        result = []
        with hub.Timeout(2):
            threads.append(hub.spawn(_child, result))
            for t in threads:
                hub.kill(t)
            hub.joinall(threads)
        assert len(result) == 0

    def test_spawn_kill_die_joinall(self):
        def _child(result):
            result.append(1)

        threads = []
        result = []
        with hub.Timeout(2):
            threads.append(hub.spawn(_child, result))
            threads.append(hub.spawn(_child, result))
            hub.sleep(0.5)
            for t in threads:
                hub.kill(t)
            hub.joinall(threads)
        assert len(result) == 2

    def test_spawn_exception_joinall(self):
        def _child():
            raise Exception("hoge")

        threads = []
        with hub.Timeout(2):
            threads.append(hub.spawn(_child))
            threads.append(hub.spawn(_child))
            hub.sleep(0.5)
            hub.joinall(threads)

    def test_event1(self):
        ev = hub.Event()
        ev.set()
        with hub.Timeout(1):
            ev.wait()  # should return immediately

    def test_event2(self):
        ev = hub.Event()
        # allow multiple sets unlike eventlet Event
        ev.set()
        ev.set()



import json
import logging
import six
import time

from datetime import datetime


class ApgwFormatter(logging.Formatter):
    LOG_TIME_FORMAT = '%Y-%m-%dT%H:%M:%S.%fZ'
    COMPONENT_NAME = 'BGPSpeaker'

    def format(self, record):
        msg = {
            'component_name': self.COMPONENT_NAME,
            'timestamp': datetime.utcfromtimestamp(
                time.time()
            ).strftime(self.LOG_TIME_FORMAT),
            'msg': six.text_type(record.msg),
            'level': record.levelname

        }

        if hasattr(record, 'log_type'):
            assert record.log_type in ('log', 'stats', 'state')
            msg['log_type'] = record.log_type
        else:
            msg['log_type'] = 'log'
        if hasattr(record, 'resource_id'):
            msg['resource_id'] = record.resource_id
        if hasattr(record, 'resource_name'):
            msg['resource_name'] = record.resource_name

        record.msg = json.dumps(msg)

        return super(ApgwFormatter, self).format(record)



# Copyright 2009 Shikhar Bhushan
# Copyright 2011 Leonidas Poulopoulos
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""This module is a thin layer of abstraction around the library. It exposes all core functionality."""

import capabilities
import operations
import transport

import logging

logger = logging.getLogger('ncclient.manager')

CAPABILITIES = [
    "urn:ietf:params:netconf:base:1.0",
    "urn:ietf:params:netconf:capability:writable-running:1.0",
    "urn:ietf:params:netconf:capability:candidate:1.0",
    "urn:ietf:params:netconf:capability:confirmed-commit:1.0",
    "urn:ietf:params:netconf:capability:rollback-on-error:1.0",
    "urn:ietf:params:netconf:capability:startup:1.0",
    "urn:ietf:params:netconf:capability:url:1.0?scheme=http,ftp,file,https,sftp",
    "urn:ietf:params:netconf:capability:validate:1.0",
    "urn:ietf:params:netconf:capability:xpath:1.0",
    "urn:liberouter:params:netconf:capability:power-control:1.0",
    "urn:ietf:params:netconf:capability:interleave:1.0"
]
"""A list of URI's representing the client's capabilities. This is used during the initial capability exchange. Modify this if you need to announce some capability not already included."""

OPERATIONS = {
    "get": operations.Get,
    "get_config": operations.GetConfig,
    "dispatch": operations.Dispatch,
    "edit_config": operations.EditConfig,
    "copy_config": operations.CopyConfig,
    "validate": operations.Validate,
    "commit": operations.Commit,
    "discard_changes": operations.DiscardChanges,
    "delete_config": operations.DeleteConfig,
    "lock": operations.Lock,
    "unlock": operations.Unlock,
    "close_session": operations.CloseSession,
    "kill_session": operations.KillSession,
    "poweroff_machine": operations.PoweroffMachine,
    "reboot_machine": operations.RebootMachine
}
"""Dictionary of method names and corresponding :class:`~ncclient.operations.RPC` subclasses. It is used to lookup operations, e.g. `get_config` is mapped to :class:`~ncclient.operations.GetConfig`. It is thus possible to add additional operations to the :class:`Manager` API."""

def connect_ssh(*args, **kwds):
    """Initialize a :class:`Manager` over the SSH transport. For documentation of arguments see :meth:`ncclient.transport.SSHSession.connect`.

    The underlying :class:`ncclient.transport.SSHSession` is created with :data:`CAPABILITIES`. It is first instructed to :meth:`~ncclient.transport.SSHSession.load_known_hosts` and then  all the provided arguments are passed directly to its implementation of :meth:`~ncclient.transport.SSHSession.connect`.
    """
    session = transport.SSHSession(capabilities.Capabilities(CAPABILITIES))
    session.load_known_hosts()
    session.connect(*args, **kwds)
    return Manager(session)

connect = connect_ssh
"Same as :func:`connect_ssh`, since SSH is the default (and currently, the only) transport."

class OpExecutor(type):

    def __new__(cls, name, bases, attrs):
        def make_wrapper(op_cls):
            def wrapper(self, *args, **kwds):
                return self.execute(op_cls, *args, **kwds)
            wrapper.func_doc = op_cls.request.func_doc
            return wrapper
        for op_name, op_cls in OPERATIONS.iteritems():
            attrs[op_name] = make_wrapper(op_cls)
        return super(OpExecutor, cls).__new__(cls, name, bases, attrs)

class Manager(object):

    """For details on the expected behavior of the operations and their parameters refer to :rfc:`4741`.

    Manager instances are also context managers so you can use it like this::

        with manager.connect("host") as m:
            # do your stuff

    ... or like this::

        m = manager.connect("host")
        try:
            # do your stuff
        finally:
            m.close_session()
    """

    __metaclass__ = OpExecutor

    def __init__(self, session, timeout=30):
        self._session = session
        self._async_mode = False
        self._timeout = timeout
        self._raise_mode = operations.RaiseMode.ALL

    def __enter__(self):
        return self

    def __exit__(self, *args):
        self.close_session()
        return False

    def __set_timeout(self, timeout):
        self._timeout = timeout

    def __set_async_mode(self, mode):
        self._async_mode = mode

    def __set_raise_mode(self, mode):
        assert(mode in (operations.RaiseMode.NONE, operations.RaiseMode.ERRORS, operations.RaiseMode.ALL))
        self._raise_mode = mode

    def execute(self, cls, *args, **kwds):
        return cls(self._session,
                   async=self._async_mode,
                   timeout=self._timeout,
                   raise_mode=self._raise_mode).request(*args, **kwds)

    def locked(self, target):
        """Returns a context manager for a lock on a datastore, where *target* is the name of the configuration datastore to lock, e.g.::

            with m.locked("running"):
                # do your stuff

        ... instead of::

            m.lock("running")
            try:
                # do your stuff
            finally:
                m.unlock("running")
        """
        return operations.LockContext(self._session, target)

    @property
    def client_capabilities(self):
        ":class:`~ncclient.capabilities.Capabilities` object representing the client's capabilities."
        return self._session._client_capabilities

    @property
    def server_capabilities(self):
        ":class:`~ncclient.capabilities.Capabilities` object representing the server's capabilities."
        return self._session._server_capabilities

    @property
    def session_id(self):
        "`session-id` assigned by the NETCONF server."
        return self._session.id

    @property
    def connected(self):
        "Whether currently connected to the NETCONF server."
        return self._session.connected

    async_mode = property(fget=lambda self: self._async_mode, fset=__set_async_mode)
    "Specify whether operations are executed asynchronously (`True`) or synchronously (`False`) (the default)."

    timeout = property(fget=lambda self: self._timeout, fset=__set_timeout)
    "Specify the timeout for synchronous RPC requests."

    raise_mode = property(fget=lambda self: self._raise_mode, fset=__set_raise_mode)
    "Specify which errors are raised as :exc:`~ncclient.operations.RPCError` exceptions. Valid values are the constants defined in :class:`~ncclient.operations.RaiseMode`. The default value is :attr:`~ncclient.operations.RaiseMode.ALL`."



# Copyright (C) 2015 Nippon Telegraph and Telephone Corporation.
# Copyright (C) 2015 YAMAMOTO Takashi <yamamoto at valinux co jp>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.


import six

from ryu.lib import addrconv


class TypeDescr(object):
    pass


class IntDescr(TypeDescr):
    def __init__(self, size):
        self.size = size

    def to_user(self, bin):
        i = 0
        for x in range(self.size):
            c = bin[:1]
            i = i * 256 + ord(c)
            bin = bin[1:]
        return i

    def from_user(self, i):
        bin = b''
        for x in range(self.size):
            bin = six.int2byte(i & 255) + bin
            i //= 256
        return bin

Int1 = IntDescr(1)
Int2 = IntDescr(2)
Int3 = IntDescr(3)
Int4 = IntDescr(4)
Int8 = IntDescr(8)
Int16 = IntDescr(16)


def _split_str(s, n):
    """
    split string into list of strings by specified number.
    """
    length = len(s)
    return [s[i:i + n] for i in range(0, length, n)]


class IntDescrMlt(TypeDescr):
    def __init__(self, length, num):
        self.length = length
        self.num = num
        self.size = length * num

    def to_user(self, bin):
        assert len(bin) == self.size
        lb = _split_str(bin, self.length)
        li = []
        for b in lb:
            i = 0
            for x in range(self.length):
                c = b[:1]
                i = i * 256 + ord(c)
                b = b[1:]
            li.append(i)
        return tuple(li)

    def from_user(self, li):
        assert len(li) == self.num
        bin = b''
        for i in li:
            b = b''
            for x in range(self.length):
                b = six.int2byte(i & 255) + b
                i //= 256
            bin += b
        return bin

Int4Double = IntDescrMlt(4, 2)


class MacAddr(TypeDescr):
    size = 6
    to_user = addrconv.mac.bin_to_text
    from_user = addrconv.mac.text_to_bin


class IPv4Addr(TypeDescr):
    size = 4
    to_user = addrconv.ipv4.bin_to_text
    from_user = addrconv.ipv4.text_to_bin


class IPv6Addr(TypeDescr):
    size = 16
    to_user = addrconv.ipv6.bin_to_text
    from_user = addrconv.ipv6.text_to_bin


class UnknownType(TypeDescr):
    import base64

    b64encode = base64.b64encode
    if six.PY3:
        @classmethod
        def to_user(cls, data):
            return cls.b64encode(data).decode('ascii')
    else:
        to_user = staticmethod(base64.b64encode)
    from_user = staticmethod(base64.b64decode)
